/*****************************************************************************
*  Copyright Statement:
*  --------------------
*  This software is protected by Copyright and the information contained
*  herein is confidential. The software may not be copied and the information
*  contained herein may not be used or disclosed except with the written
*  permission of MediaTek Inc. (C) 2010
*
*  BY OPENING THIS FILE, BUYER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
*  THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
*  RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO BUYER ON
*  AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES,
*  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
*  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.
*  NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH RESPECT TO THE
*  SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY, INCORPORATED IN, OR
*  SUPPLIED WITH THE MEDIATEK SOFTWARE, AND BUYER AGREES TO LOOK ONLY TO SUCH
*  THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO. MEDIATEK SHALL ALSO
*  NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE RELEASES MADE TO BUYER'S
*  SPECIFICATION OR TO CONFORM TO A PARTICULAR STANDARD OR OPEN FORUM.
*
*  BUYER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S ENTIRE AND CUMULATIVE
*  LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE RELEASED HEREUNDER WILL BE,
*  AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE MEDIATEK SOFTWARE AT ISSUE,
*  OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE CHARGE PAID BY BUYER TO
*  MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
*
*  THE TRANSACTION CONTEMPLATED HEREUNDER SHALL BE CONSTRUED IN ACCORDANCE
*  WITH THE LAWS OF THE STATE OF CALIFORNIA, USA, EXCLUDING ITS CONFLICT OF
*  LAWS PRINCIPLES.  ANY DISPUTES, CONTROVERSIES OR CLAIMS ARISING THEREOF AND
*  RELATED THERETO SHALL BE SETTLED BY ARBITRATION IN SAN FRANCISCO, CA, UNDER
*  THE RULES OF THE INTERNATIONAL CHAMBER OF COMMERCE (ICC).
*
*****************************************************************************/

/*****************************************************************************
 *
 * Filename:
 * ---------
 *   image_sensor_SP0827_SERIAL.c  V1.0
 *
 * Project:
 * --------
 *   Maui_sw
 *
 * Description:
 * ------------
 *   Image sensor driver function
 *
 * Author:
 * -------
 *   superpix xg.
 *
 *============================================================================
 *             HISTORY
 * Below this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *------------------------------------------------------------------------------
 * $Revision:$
 * $Modtime:$
 * $Log:$
 *
 *
 *
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *============================================================================
 ****************************************************************************/
#include "drv_comm.h"
#include "isp_comm_if.h"
#include "camera_sccb.h"
#include "image_sensor.h"
#include "cal_comm_def.h"
#include "isp_if.h"
#define SP0827_SERIAL_SPI_CLOCK_PIN                          59
#define SP0827_SERIAL_SPI_DATA_PIN                            47
#define SP0827_SERIAL_POWERDOWN_PIN                           57
#define SP0827_SERIAL_I2C_CLOCK_PIN                          22
#define SP0827_SERIAL_I2C_DATA_PIN                          23

#if defined(DUAL_CAMERA_SUPPORT)
extern kal_bool camera_front_or_back;
#endif
#define  SP0827_DAYLIGHT_MAXGAIN             0x70
#define  SP0827_NIGHT_MAXGAIN                0xa0
#define __DUAL_Camera_SP0827__ // SP0827 use to Sub_Camera 

static struct
{
  kal_bool BypassAe;
  kal_bool BypassAwb;
  
  kal_bool CapState; /* KAL_TRUE: in capture state, else in preview state */
  kal_bool PvMode; /* KAL_TRUE: in preview mode, else in capture mode */
  kal_uint8 BandingFreq; /* SP0827_SERIAL_50HZ or SP0827_SERIAL_60HZ for 50Hz/60Hz */
  
  kal_uint32 InternalClock; /* internal clock which using process pixel(for exposure) */
  kal_uint32 Pclk; /* output clock which output to baseband */
  kal_uint32 Gain; /* base on 0x40 */
  kal_uint32 Shutter; /* unit is (linelength / internal clock) s */
  
  kal_uint32 FrameLength; /* total line numbers in one frame(include dummy line) */
  kal_uint32 LineLength; /* total pixel numbers in one line(include dummy pixel) */
  
  IMAGE_SENSOR_INDEX_ENUM SensorIdx;
  sensor_data_struct *NvramData;
  
  kal_uint16 MinFrameRate;	/* 150 means 15.0fps. */
  kal_uint16 MaxFrameRate;	/* 300 means 30.0fps. */
  
  kal_uint32 dummy_lines;
  kal_uint32 dummy_pixels;
  
  kal_uint32 curr_frame_time;	/* It's get from the TG Int. */
	
  kal_bool sccb_opened;			/* The I2C can work when sccb_opened is KAL_TRUE */
} SP0827_SERIALSensor;

kal_bool video_mode = KAL_FALSE;
kal_uint8 SP0827_SERIAL_CAM_BANDING_50HZ = CAM_BANDING_60HZ;
kal_bool SP0827_SERIAL_CAM_nightmode = KAL_FALSE;


/*************************************************************************
* FUNCTION
*   SP0827_SERIALSetMirror
*
* DESCRIPTION
*   This function set the mirror to the CMOS sensor
*
* PARAMETERS
*   Mirror
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALSetMirror(kal_uint8 Mirror)
{
#if 0
  kal_uint8 CntrB;
  
  switch (Mirror)
  {
  case IMAGE_SENSOR_MIRROR_NORMAL:
  	CamWriteCmosSensor(0x31, 0x10); 
	CamWriteCmosSensor(0x5f, 0x01); 
    break;
  case IMAGE_SENSOR_MIRROR_H:
    CamWriteCmosSensor(0x31, 0x30);
	CamWriteCmosSensor(0x5f, 0x04); 
    break;
  case IMAGE_SENSOR_MIRROR_V:
    CamWriteCmosSensor(0x31, 0x50);
	CamWriteCmosSensor(0x5f, 0x0b); 
    break;
  case IMAGE_SENSOR_MIRROR_HV:
    CamWriteCmosSensor(0x31, 0x70); 
	CamWriteCmosSensor(0x5f, 0x0e); 
    break;
  default:
    /* no need to do error handling */
    break;
  }
  #endif
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSetClock
*
* DESCRIPTION
*   This function set sensor internal clock and output clock
*
* PARAMETERS
*   Clk: internal clock
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALSetClock(kal_uint32 InternalClock)
{
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALWriteShutter
*
* DESCRIPTION
*   This function apply shutter to sensor
*
* PARAMETERS
*   Shutter: integration time, unit: (linelength / internal clock) s
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALWriteShutter(kal_uint32 Shutter)
{
    // TODO: set integration time here
	//CamWriteCmosSensor(0x03, Shutter >> 8);
	//CamWriteCmosSensor(0x04, Shutter);    
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALReadShutter
*
* DESCRIPTION
*   This function get shutter from sensor
*
* PARAMETERS
*   None
*
* RETURNS
*   Shutter: integration time, unit: (linelength / internal clock) s
*
* LOCAL AFFECTED
*
*************************************************************************/
static kal_uint32 SP0827_SERIALReadShutter(void)
{
    // TODO: read integration time here
	//return (CamReadCmosSensor(0x03) << 8)|CamReadCmosSensor(0x04);
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALWriteGain
*
* DESCRIPTION
*   This function apply global gain to sensor
*
* PARAMETERS
*   Gain: base on 0x40
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALWriteGain(kal_uint32 Gain)
{
    // TODO: set global gain here
    
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALReadGain
*
* DESCRIPTION
*   This function get global gain from sensor
*
* PARAMETERS
*   None
*
* RETURNS
*   Gain: base on 0x40
*
* LOCAL AFFECTED
*
*************************************************************************/
static kal_uint32 SP0827_SERIALReadGain(void)
{
    // TODO: read global gain here
    
  //  return 0x40;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALAeEnable
*
* DESCRIPTION
*   disable/enable AE
*
* PARAMETERS
*   Enable
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALAeEnable(kal_bool Enable)
{
#if 1
	if (SP0827_SERIALSensor.BypassAe)
	{
		Enable = KAL_FALSE;
	}
	
	// TODO: enable or disable AE here
	{
		kal_uint16 temp_ae_reg = CamReadCmosSensor(0x32);
		if (Enable)
		{
	    		CamWriteCmosSensor(0x32, (temp_ae_reg | 0x01));
		}
		else
		{
	    		CamWriteCmosSensor(0x32, (temp_ae_reg & (~0x01))); /* Turn OFF AEC/AGC*/
		}
	}
	#endif
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALAwbEnable
*
* DESCRIPTION
*   disable/enable awb
*
* PARAMETERS
*   Enable
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALAwbEnable(kal_bool Enable)
{
#if 1
	if (SP0827_SERIALSensor.BypassAwb)
	{
		Enable = KAL_FALSE;
	}
	
	// TODO: enable or disable AWB here
	{
		kal_uint16 temp_awb_reg = CamReadCmosSensor(0x32);
		if (Enable)
		{
	    		CamWriteCmosSensor(0x32, (temp_awb_reg | 0x08));
		}
		else
		{
	    		CamWriteCmosSensor(0x32, (temp_awb_reg & (~0x08))); /* Turn OFF AWB*/
		}
	}
	#endif
}


/*************************************************************************
* FUNCTION
*   SP0827_SERIALSetBandingStep
*
* DESCRIPTION
*   This function is to calculate & set shutter step register .
*
*************************************************************************/
static void SP0827_SERIALSetBandingStep(CAL_CAMERA_BANDING_FREQ_ENUM BandingFreq)
{
    // TODO: set banding step here
    
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSetDummy
*
* DESCRIPTION
*   This function set the dummy pixels(Horizontal Blanking) & dummy lines(Vertical Blanking), it can be
*   used to adjust the frame rate or gain more time for back-end process.
*
* PARAMETERS
*   DummyPixel
*   DummyLine
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALSetDummy(kal_uint32 DummyPixel, kal_uint32 DummyLine)
{
#if 0
    kal_uint32 LineLength, FrameLength;

    if (SP0827_SERIALSensor.PvMode) /* preview size output mode */
    {
        LineLength = DummyPixel + SP0827_SERIAL_PV_PERIOD_PIXEL_NUMS;
        FrameLength = DummyLine + SP0827_SERIAL_PV_PERIOD_LINE_NUMS;
    }
    else
    {
        LineLength = DummyPixel + SP0827_SERIAL_FULL_PERIOD_PIXEL_NUMS;
        FrameLength = DummyLine + SP0827_SERIAL_FULL_PERIOD_LINE_NUMS;
    }

    // TODO: set linelength/framelength or dummy pixel/line and return right value here
    {
        
    }
#endif
    /* config banding step or base shutter */
  //  if (SP0827_SERIALSensor.PvMode) /* preview size output mode */
  //  {
   //     SP0827_SERIALSetBandingStep();
   // }
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSetMinFps
*
* DESCRIPTION
*   This function calculate & set min frame rate
*
* PARAMETERS
*   Fps: min frame rate, base on SP0827_SERIAL_FPS(x)
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALSetMinFps(kal_uint16 Fps)
{
    // TODO: set max exposure time or max AE index here
    
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSetMaxFps
*
* DESCRIPTION
*   This function calculate & set max frame rate
*
* PARAMETERS
*   Fps: max frame rate, base on SP0827_SERIAL_FPS(x)
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALSetMaxFps(kal_uint16 Fps)
{
}



/*************************************************************************
* FUNCTION
*   SP0827_SERIALSetNightMode
*
* DESCRIPTION
*   This function switch on/off night mode of SP0827_SERIAL.
*
*************************************************************************/
static void SP0827_SERIALSetVideoFps(kal_uint16 Fps)
{
    // TODO: fix, set max exposure time or max AE index here
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALNightMode
*
* DESCRIPTION
*   This function set night mode to sensor.
*
* PARAMETERS
*   Enable
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALNightMode(kal_bool videomode,kal_uint8 banding,kal_bool nightmode)
{
CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x80);
	if (KAL_FALSE ==nightmode) //daylight
	{	
		if(KAL_TRUE == videomode)	
			{
				if(banding == CAM_BANDING_50HZ)
				{
					//Video record daylight 24M 50hz fix 14FPS 
					CamWriteCmosSensor(0x06,0x00);			 
	        CamWriteCmosSensor(0x09,0x00);			 
	        CamWriteCmosSensor(0x0a,0x2c);  			 
	         				   
	        CamWriteCmosSensor(0x97,0x32);//base			 
	        CamWriteCmosSensor(0x98,0x00);			 
	                        		  	   
	        CamWriteCmosSensor(0x70,0x5e);//exp			 
	        CamWriteCmosSensor(0x71,0x01);			 
	        CamWriteCmosSensor(0x72,0x32);			 
	        CamWriteCmosSensor(0x73,0x00);			 
	        CamWriteCmosSensor(0x78,0x32);			 
	        CamWriteCmosSensor(0x79,0x00);			 
	        CamWriteCmosSensor(0x7a,0x01);			 
	        CamWriteCmosSensor(0x7b,0x00);
			
		  CamWriteCmosSensor(0x74,SP0827_DAYLIGHT_MAXGAIN);	
	         CamWriteCmosSensor(0x7c,SP0827_DAYLIGHT_MAXGAIN);
			 
		  CamWriteCmosSensor(0x64,SP0827_DAYLIGHT_MAXGAIN);
               CamWriteCmosSensor(0x65,SP0827_DAYLIGHT_MAXGAIN);
			   
		  CamWriteCmosSensor(0x66,0x5e);
		  CamWriteCmosSensor(0x67,0x01);
		  CamWriteCmosSensor(0x68,0x5e);
		  CamWriteCmosSensor(0x69,0x01);
	
					dbg_print(" video 50Hz normal\r\n");
				}
				else if(banding == CAM_BANDING_60HZ)
				{
					//Video record daylight 24M 60hz fix 14FPS 
					CamWriteCmosSensor(0x06,0x00);				
	        CamWriteCmosSensor(0x09,0x00);				
	        CamWriteCmosSensor(0x0a,0x40);  				
	         			  	        		
	        CamWriteCmosSensor(0x97,0x27);//base			
	        CamWriteCmosSensor(0x98,0x00);				
	                        	  	  			
	        CamWriteCmosSensor(0x70,0x38);//exp			
	        CamWriteCmosSensor(0x71,0x01);				
	        CamWriteCmosSensor(0x72,0x27);				
	        CamWriteCmosSensor(0x73,0x00);				
	        CamWriteCmosSensor(0x78,0x27);				
	        CamWriteCmosSensor(0x79,0x00);				
	        CamWriteCmosSensor(0x7a,0x01);				
	        CamWriteCmosSensor(0x7b,0x00);    
			            
		  CamWriteCmosSensor(0x74,SP0827_DAYLIGHT_MAXGAIN);	
	         CamWriteCmosSensor(0x7c,SP0827_DAYLIGHT_MAXGAIN);
			 
		  CamWriteCmosSensor(0x64,SP0827_DAYLIGHT_MAXGAIN);
               CamWriteCmosSensor(0x65,SP0827_DAYLIGHT_MAXGAIN);
			   
		  CamWriteCmosSensor(0x66,0x38);
		  CamWriteCmosSensor(0x67,0x01);
		  CamWriteCmosSensor(0x68,0x38);
		  CamWriteCmosSensor(0x69,0x01);
					dbg_print(" video 60Hz normal\r\n");
				}
			}
		else
			{
				if(banding == CAM_BANDING_50HZ)
				{
				//caprure preview daylight 24M 50hz 15-8FPS 	
					CamWriteCmosSensor(0x06,0x00);			 
	        CamWriteCmosSensor(0x09,0x00);			 
	        CamWriteCmosSensor(0x0a,0x2c);  			 
	         				   
	        CamWriteCmosSensor(0x97,0x32);//base			 
	        CamWriteCmosSensor(0x98,0x00);			 
	                        		  	   
	        CamWriteCmosSensor(0x70,0x58);//exp			 
	        CamWriteCmosSensor(0x71,0x02);			 
	        CamWriteCmosSensor(0x72,0x32);			 
	        CamWriteCmosSensor(0x73,0x00);			 
	        CamWriteCmosSensor(0x78,0x32);			 
	        CamWriteCmosSensor(0x79,0x00);			 
	        CamWriteCmosSensor(0x7a,0x01);			 
	        CamWriteCmosSensor(0x7b,0x00);

		 CamWriteCmosSensor(0x74,SP0827_DAYLIGHT_MAXGAIN);	
	         CamWriteCmosSensor(0x7c,SP0827_DAYLIGHT_MAXGAIN);
			 
		  CamWriteCmosSensor(0x64,SP0827_DAYLIGHT_MAXGAIN);
               CamWriteCmosSensor(0x65,SP0827_DAYLIGHT_MAXGAIN);
			   
		  CamWriteCmosSensor(0x66,0x58);
		  CamWriteCmosSensor(0x67,0x02);
		  CamWriteCmosSensor(0x68,0x58);
		  CamWriteCmosSensor(0x69,0x02);
					dbg_print(" priview 50Hz normal\r\n");
				 
				}
				else if(banding == CAM_BANDING_60HZ)
				{
					//caprure preview daylight 24M 60hz 15-8FPS 		
					CamWriteCmosSensor(0x06,0x00);			 
	        CamWriteCmosSensor(0x09,0x00);			 
	        CamWriteCmosSensor(0x0a,0x40);  			 
	         				   
	        CamWriteCmosSensor(0x97,0x27);//base			 
	        CamWriteCmosSensor(0x98,0x00);			 
	                        		  	   
	        CamWriteCmosSensor(0x70,0x49);//exp			 
	        CamWriteCmosSensor(0x71,0x02);			 
	        CamWriteCmosSensor(0x72,0x27);			 
	        CamWriteCmosSensor(0x73,0x00);			 
	        CamWriteCmosSensor(0x78,0x27);			 
	        CamWriteCmosSensor(0x79,0x00);			 
	        CamWriteCmosSensor(0x7a,0x01);			 
	        CamWriteCmosSensor(0x7b,0x00);

		CamWriteCmosSensor(0x74,SP0827_DAYLIGHT_MAXGAIN);	
	         CamWriteCmosSensor(0x7c,SP0827_DAYLIGHT_MAXGAIN);
			 
		  CamWriteCmosSensor(0x64,SP0827_DAYLIGHT_MAXGAIN);
               CamWriteCmosSensor(0x65,SP0827_DAYLIGHT_MAXGAIN);
			   
		  CamWriteCmosSensor(0x66,0x49);
		  CamWriteCmosSensor(0x67,0x02);
		  CamWriteCmosSensor(0x68,0x49);
		  CamWriteCmosSensor(0x69,0x02);
					dbg_print(" priview 60Hz normal\r\n");
				
				 
				}
			}	
		}
	else //night mode
	{
			if(KAL_TRUE == videomode)		
			{
				if(banding == CAM_BANDING_50HZ)
				{
					//Video record night 24M 50hz  fix 9FPS 
				CamWriteCmosSensor(0x06,0x00);			 
	        CamWriteCmosSensor(0x09,0x00);			 
	        CamWriteCmosSensor(0x0a,0xe4);  			 
	         				   
	        CamWriteCmosSensor(0x97,0x1f);//base			 
	        CamWriteCmosSensor(0x98,0x00);			 
	                        		  	   
	        CamWriteCmosSensor(0x70,0x55);//exp			 
	        CamWriteCmosSensor(0x71,0x01);			 
	        CamWriteCmosSensor(0x72,0x1f);			 
	        CamWriteCmosSensor(0x73,0x00);			 
	        CamWriteCmosSensor(0x78,0x1f);			 
	        CamWriteCmosSensor(0x79,0x00);			 
	        CamWriteCmosSensor(0x7a,0x01);			 
	        CamWriteCmosSensor(0x7b,0x00);

		CamWriteCmosSensor(0x74,SP0827_NIGHT_MAXGAIN);	
	         CamWriteCmosSensor(0x7c,SP0827_NIGHT_MAXGAIN);
			 
		  CamWriteCmosSensor(0x64,SP0827_NIGHT_MAXGAIN);
               CamWriteCmosSensor(0x65,SP0827_NIGHT_MAXGAIN);
			   
		  CamWriteCmosSensor(0x66,0x55);
		  CamWriteCmosSensor(0x67,0x01);
		  CamWriteCmosSensor(0x68,0x55);
		  CamWriteCmosSensor(0x69,0x01);
					dbg_print(" video 50Hz night\r\n"); 			
				}
				else if(banding == CAM_BANDING_60HZ)
				{
					//Video record night 24M 60hz fix 9FPS 
					CamWriteCmosSensor(0x06,0x00);			 
	        CamWriteCmosSensor(0x09,0x00);			 
	        CamWriteCmosSensor(0x0a,0xe1);  			 
	         				   
	        CamWriteCmosSensor(0x97,0x1a);//base			 
	        CamWriteCmosSensor(0x98,0x00);			 
	                        		  	   
	        CamWriteCmosSensor(0x70,0x52);//exp			 
	        CamWriteCmosSensor(0x71,0x01);			 
	        CamWriteCmosSensor(0x72,0x1a);			 
	        CamWriteCmosSensor(0x73,0x00);			 
	        CamWriteCmosSensor(0x78,0x1a);			 
	        CamWriteCmosSensor(0x79,0x00);			 
	        CamWriteCmosSensor(0x7a,0x01);			 
	        CamWriteCmosSensor(0x7b,0x00);

		 CamWriteCmosSensor(0x74,SP0827_NIGHT_MAXGAIN);	
	         CamWriteCmosSensor(0x7c,SP0827_NIGHT_MAXGAIN);
			 
		  CamWriteCmosSensor(0x64,SP0827_NIGHT_MAXGAIN);
               CamWriteCmosSensor(0x65,SP0827_NIGHT_MAXGAIN);
			   
		  CamWriteCmosSensor(0x66,0x52);
		  CamWriteCmosSensor(0x67,0x01);
		  CamWriteCmosSensor(0x68,0x52);
		  CamWriteCmosSensor(0x69,0x01);
					dbg_print(" video 60Hz night\r\n"); 
				}
			}
		   else
			{
				dbg_print(" banding=%x\r\n",banding);
				if(banding == CAM_BANDING_50HZ)
				{
					//caprure preview night 24M 50hz 15-5FPS	
					CamWriteCmosSensor(0x06,0x00);			 
	        CamWriteCmosSensor(0x09,0x00);			 
	        CamWriteCmosSensor(0x0a,0x2c);  			 
	         				   
	        CamWriteCmosSensor(0x97,0x32);//base			 
	        CamWriteCmosSensor(0x98,0x00);			 
	                        		  	   
	        CamWriteCmosSensor(0x70,0xe8);//exp			 
	        CamWriteCmosSensor(0x71,0x03);			 
	        CamWriteCmosSensor(0x72,0x32);			 
	        CamWriteCmosSensor(0x73,0x00);			 
	        CamWriteCmosSensor(0x78,0x32);			 
	        CamWriteCmosSensor(0x79,0x00);			 
	        CamWriteCmosSensor(0x7a,0x01);			 
	        CamWriteCmosSensor(0x7b,0x00);
                
		CamWriteCmosSensor(0x74,SP0827_NIGHT_MAXGAIN);	
	         CamWriteCmosSensor(0x7c,SP0827_NIGHT_MAXGAIN);
			 
		  CamWriteCmosSensor(0x64,SP0827_NIGHT_MAXGAIN);
               CamWriteCmosSensor(0x65,SP0827_NIGHT_MAXGAIN);
			   
		  CamWriteCmosSensor(0x66,0xe8);
		  CamWriteCmosSensor(0x67,0x03);
		  CamWriteCmosSensor(0x68,0xe8);
		  CamWriteCmosSensor(0x69,0x03);
					dbg_print(" priview 50Hz night\r\n");	
				}
				else if(banding == CAM_BANDING_60HZ)
				{
					//caprure preview night 24M 60hz 15-5FPS		
					CamWriteCmosSensor(0x06,0x00);			 
	        CamWriteCmosSensor(0x09,0x00);			 
	        CamWriteCmosSensor(0x0a,0x40);  			 
	         				   
	        CamWriteCmosSensor(0x97,0x27);//base			 
	        CamWriteCmosSensor(0x98,0x00);			 
	                        		  	   
	        CamWriteCmosSensor(0x70,0xa8);//exp			 
	        CamWriteCmosSensor(0x71,0x03);			 
	        CamWriteCmosSensor(0x72,0x27);			 
	        CamWriteCmosSensor(0x73,0x00);			 
	        CamWriteCmosSensor(0x78,0x27);			 
	        CamWriteCmosSensor(0x79,0x00);			 
	        CamWriteCmosSensor(0x7a,0x01);			 
	        CamWriteCmosSensor(0x7b,0x00);

		 CamWriteCmosSensor(0x74,SP0827_NIGHT_MAXGAIN);	
	         CamWriteCmosSensor(0x7c,SP0827_NIGHT_MAXGAIN);
			 
		  CamWriteCmosSensor(0x64,SP0827_NIGHT_MAXGAIN);
               CamWriteCmosSensor(0x65,SP0827_NIGHT_MAXGAIN);
			   
		  CamWriteCmosSensor(0x66,0xa8);
		  CamWriteCmosSensor(0x67,0x03);
		  CamWriteCmosSensor(0x68,0xa8);
		  CamWriteCmosSensor(0x69,0x03);
					dbg_print(" priview 60Hz night\r\n");	
				}
			}
	}
	
	
	
}


/*************************************************************************
* FUNCTION
*   SP0827_SERIALEv
*
* DESCRIPTION
*   Exposure setting.
*
* PARAMETERS
*   In, Out
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/

static MM_ERROR_CODE_ENUM SP0827_SERIALEv(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Exposure = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;

    Exposure->IsSupport = KAL_TRUE;
    Exposure->ItemCount = 10;
    Exposure->SupportItem[0] = CAM_EV_NEG_4_3;
    Exposure->SupportItem[1] = CAM_EV_NEG_3_3;
    Exposure->SupportItem[2] = CAM_EV_NEG_2_3;
    Exposure->SupportItem[3] = CAM_EV_NEG_1_3;
    Exposure->SupportItem[4] = CAM_EV_ZERO;
    Exposure->SupportItem[5] = CAM_EV_POS_1_3;
    Exposure->SupportItem[6] = CAM_EV_POS_2_3;
    Exposure->SupportItem[7] = CAM_EV_POS_3_3;
    Exposure->SupportItem[8] = CAM_EV_POS_4_3;
    Exposure->SupportItem[9] = CAM_EV_NIGHT_SHOT;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {  
 #if(defined(SP0829_SERIAL_YUV))
  CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x86);
   switch (In->FeatureSetValue)
    {

		case CAM_EV_NEG_4_3:    // -4 EV                                
			CamWriteCmosSensor(0x78,0xc0);     
		                                                     
		    break;                                               
		case CAM_EV_NEG_3_3:    // -3 EV                                 
			CamWriteCmosSensor(0x78,0xd0);     
			break;                                               
		                                         
		case CAM_EV_NEG_2_3:    // -2 EV                         
			CamWriteCmosSensor(0x78,0xe0);     
			break;                                               
		                                                     
		case CAM_EV_NEG_1_3:    // -1 EV                         
			CamWriteCmosSensor(0x78,0xf0);     
			break;                                               
		                                                     
		case CAM_EV_ZERO:    // +0 EV                            
			CamWriteCmosSensor(0x78,0x00);         
			break;                                               
		                                                     
		case CAM_EV_POS_1_3:   // +1 EV                         
			CamWriteCmosSensor(0x78,0x10);    
			break;                                               
		                                                     
		case CAM_EV_POS_2_3:    // +2 EV                         
			CamWriteCmosSensor(0x78,0x20);    
			break;                                               
		                                                     
		case CAM_EV_POS_3_3:    // +3 EV                         
			CamWriteCmosSensor(0x78,0x30);         
			break;                                               
		                                                     
		case CAM_EV_POS_4_3:    // +4EV                          
			CamWriteCmosSensor(0x78,0x40);          
			break;                                               

        default:
            return KAL_FALSE;    
    }

 #elif(defined(SP0A18_YUV)||defined(ALL_YUV))
  CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x42);
    switch (In->FeatureSetValue)
    {
  		case CAM_EV_NEG_4_3:    // -4 EV                                
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0xc0);     
		                                                     
		    break;                                               
		case CAM_EV_NEG_3_3:    // -3 EV                                 
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0xd0);    
			break;                                               
		                                         
		case CAM_EV_NEG_2_3:    // -2 EV                         
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0xe0);     
			break;                                               
		                                                     
		case CAM_EV_NEG_1_3:    // -1 EV                         
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0xf0);     
			break;                                               
		                                                     
		case CAM_EV_ZERO:    // +0 EV                            
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0x00);         
			break;                                               
		                                                     
		case CAM_EV_POS_1_3:   // +1 EV                         
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0x10);    
			break;                                               
		                                                     
		case CAM_EV_POS_2_3:    // +2 EV                         
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0x20);    
			break;                                               
		                                                     
		case CAM_EV_POS_3_3:    // +3 EV                         
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0x30);         
			break;                                               
		                                                     
		case CAM_EV_POS_4_3:    // +4EV                          
			CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0xdc, 0x40);        
			break; 
    default:
   
      return MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    }
#endif
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALContrast
*
* DESCRIPTION
*   Contrast setting.
*
* PARAMETERS
*   In, Out
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0827_SERIALContrast(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
#if 0
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Contrast = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;

    Contrast->IsSupport = KAL_TRUE;
    Contrast->ItemCount = 3;
    Contrast->SupportItem[0] = CAM_CONTRAST_HIGH;
    Contrast->SupportItem[1] = CAM_CONTRAST_MEDIUM;
    Contrast->SupportItem[2] = CAM_CONTRAST_LOW;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
    switch (In->FeatureSetValue)
    {
    case CAM_CONTRAST_HIGH:
	//	CamWriteCmosSensor(0x5c, 0x90);
      break;
    case CAM_CONTRAST_MEDIUM:
	//	CamWriteCmosSensor(0x5c, 0x80);
      break;
    case CAM_CONTRAST_LOW:
	//	CamWriteCmosSensor(0x5c, 0x70);
      break;
    default:
      return MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    }
  }
  return MM_ERROR_NONE;
  #endif
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSaturation
*
* DESCRIPTION
*   Saturation setting.
*
* PARAMETERS
*   In, Out
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0827_SERIALSaturation(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
#if 0
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Saturation = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;
    // TODO: add supported saturation enum here
    Saturation->IsSupport = KAL_FALSE;
    //Saturation->ItemCount = 3;
    //Saturation->SupportItem[0] = CAM_SATURATION_HIGH;
    //Saturation->SupportItem[1] = CAM_SATURATION_MEDIUM;
    //Saturation->SupportItem[2] = CAM_SATURATION_LOW;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
    // TODO: adjust saturation here
    //switch (In->FeatureSetValue)
    //{
    //case CAM_SATURATION_HIGH:
    //  break;
    //case CAM_SATURATION_MEDIUM:
    //  break;
    //case CAM_SATURATION_LOW:
    //  break;
    //default:
    //  return MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    //}
  }
  return MM_ERROR_NONE;
  #endif
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSharpness
*
* DESCRIPTION
*   Sharpness setting.
*
* PARAMETERS
*   In, Out
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0827_SERIALSharpness(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
#if 0
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Sharpness = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;
    // TODO: add supported sharpness enum here
    Sharpness->IsSupport = KAL_FALSE;
    //Sharpness->ItemCount = 3;
    //Sharpness->SupportItem[0] = CAM_SHARPNESS_HIGH;
    //Sharpness->SupportItem[1] = CAM_SHARPNESS_MEDIUM;
    //Sharpness->SupportItem[2] = CAM_SHARPNESS_LOW;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
    // TODO: adjust sharpness here
    //switch (In->FeatureSetValue)
    //{
    //case CAM_SHARPNESS_HIGH:
    //  break;
    //case CAM_SHARPNESS_MEDIUM:
    //  break;
    //case CAM_SHARPNESS_LOW:
    //  break;
    //default:
    //  return MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    //}
  }
  return MM_ERROR_NONE;
  #endif
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALWb
*
* DESCRIPTION
*   Wb setting.
*
* PARAMETERS
*   In, Out
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/

static MM_ERROR_CODE_ENUM SP0827_SERIALWb(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Wb = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;

    Wb->IsSupport = KAL_TRUE;
    Wb->ItemCount = 6;
    Wb->SupportItem[0] = CAM_WB_AUTO;
    Wb->SupportItem[1] = CAM_WB_CLOUD;
    Wb->SupportItem[2] = CAM_WB_DAYLIGHT;
    Wb->SupportItem[3] = CAM_WB_INCANDESCENCE;
    Wb->SupportItem[4] = CAM_WB_FLUORESCENT;
    Wb->SupportItem[5] = CAM_WB_TUNGSTEN;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
  #if(defined(SP0829_SERIAL_YUV))
CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x86);
  switch (In->FeatureSetValue)
    {
        case CAM_WB_AUTO:   // enable AWB                                              
            // CAM_WB_AUTO mode should not update R/G/B gains                                       
			CamWriteCmosSensor(0xc5, 0xc4);           
			CamWriteCmosSensor(0xc6, 0x9f);  
			 CamWriteCmosSensor(0xe7, 0x03);
	           
			CamWriteCmosSensor(0x32, 0x08);           
            return KAL_TRUE;                                                   
                                                                               
        case CAM_WB_CLOUD:                                                     
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xc5, 0xe5);                       
            CamWriteCmosSensor(0xc6, 0x80);                       
            break;                                                             
                                                                               
        case CAM_WB_DAYLIGHT:                                                  
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xc5, 0xc5);                       
            CamWriteCmosSensor(0xc6, 0x8a);                       
            break;                                                             
                                                                               
        case CAM_WB_INCANDESCENCE:                                             
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xc5, 0x7f);                       
            CamWriteCmosSensor(0xc6, 0x88);                       
            break;                                                             
                                                                               
        case CAM_WB_FLUORESCENT:                                               
             CamWriteCmosSensor(0x32, 0x00);  // disable AWB      
            CamWriteCmosSensor(0xc5, 0x9c);                       
            CamWriteCmosSensor(0xc6, 0xaf);                       
            break;                                                             
                                                                               
        case CAM_WB_TUNGSTEN:                                                  
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xc5, 0xa8);                       
            CamWriteCmosSensor(0xc6, 0xb7);                       
            break;                                                             


        case CAM_WB_MANUAL:
        default:
            return KAL_FALSE;
    }
  #elif(defined(SP0A18_YUV)||defined(ALL_YUV))
  CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x42);
    switch (In->FeatureSetValue)
    {
        case CAM_WB_AUTO:   // enable AWB                                              
            // CAM_WB_AUTO mode should not update R/G/B gains  
         CamWriteCmosSensor(0xfd, 0x01);                                      
		 CamWriteCmosSensor(0x28, 0xc4);           
		 CamWriteCmosSensor(0x29, 0x9e);  		 
	         CamWriteCmosSensor(0xfd, 0x00); 
		 CamWriteCmosSensor(0x32, 0x08);           
            return KAL_TRUE;                                                   
                                                                               
        case CAM_WB_CLOUD:  
            CamWriteCmosSensor(0xfd, 0x00);                                                   
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xfd, 0x01);                     
            CamWriteCmosSensor(0x28, 0xe2); 
            CamWriteCmosSensor(0x29, 0x82); 
            CamWriteCmosSensor(0xfd, 0x00); 
                                  
            break;                                                             
                                                                               
        case CAM_WB_DAYLIGHT:                                                  
            CamWriteCmosSensor(0xfd, 0x00);                                                   
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xfd, 0x01);                     
            CamWriteCmosSensor(0x28, 0xc1); 
            CamWriteCmosSensor(0x29, 0x88); 
            CamWriteCmosSensor(0xfd, 0x00);                       
            break;                                                             
                                                                               
        case CAM_WB_INCANDESCENCE:                                             
            CamWriteCmosSensor(0xfd, 0x00);                                                   
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xfd, 0x01);                     
            CamWriteCmosSensor(0x28, 0x7b); 
            CamWriteCmosSensor(0x29, 0xd3); 
            CamWriteCmosSensor(0xfd, 0x00);                       
            break;                                                             
                                                                               
        case CAM_WB_FLUORESCENT:                                               
            CamWriteCmosSensor(0xfd, 0x00);                                                   
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xfd, 0x01);                     
            CamWriteCmosSensor(0x28, 0xb4); 
            CamWriteCmosSensor(0x29, 0xc4); 
            CamWriteCmosSensor(0xfd, 0x00);                       
            break;                                                             
                                                                               
        case CAM_WB_TUNGSTEN:                                                  
            CamWriteCmosSensor(0xfd, 0x00);                                                   
            CamWriteCmosSensor(0x32, 0x00);  // disable AWB       
            CamWriteCmosSensor(0xfd, 0x01);                     
            CamWriteCmosSensor(0x28, 0xae); 
            CamWriteCmosSensor(0x29, 0xcc); 
            CamWriteCmosSensor(0xfd, 0x00);                       
            break;                                                             

        case CAM_WB_MANUAL:
        default:
            return KAL_FALSE;
    }
#endif
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALEffect
*
* DESCRIPTION
*   Effect setting.
*
* PARAMETERS
*   In, Out
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/

static MM_ERROR_CODE_ENUM SP0827_SERIALEffect(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT Effect = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;

    Effect->IsSupport = KAL_TRUE;
    Effect->ItemCount = 6;
    Effect->SupportItem[0] = CAM_EFFECT_ENC_NORMAL;
    Effect->SupportItem[1] = CAM_EFFECT_ENC_GRAYSCALE;
    Effect->SupportItem[2] = CAM_EFFECT_ENC_SEPIA;
    Effect->SupportItem[3] = CAM_EFFECT_ENC_SEPIAGREEN;
    Effect->SupportItem[4] = CAM_EFFECT_ENC_SEPIABLUE;
    Effect->SupportItem[5] = CAM_EFFECT_ENC_COLORINV;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
  #if(defined(SP0829_SERIAL_YUV))
  CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x86);
  switch (In->FeatureSetValue)
  {
       case CAM_EFFECT_ENC_NORMAL:                                                            
           CamWriteCmosSensor(0x62, 0x00);                                
		   CamWriteCmosSensor(0x63, 0x80);                    
		   CamWriteCmosSensor(0x64, 0x80);                    
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_GRAYSCALE:                                                  
           CamWriteCmosSensor(0x62, 0x08);                                
		   CamWriteCmosSensor(0x63, 0x80);                    
		   CamWriteCmosSensor(0x64, 0x80);                    
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_SEPIA:                                                      
           CamWriteCmosSensor(0x62, 0x04);                                
		CamWriteCmosSensor(0x63, 0xc0);                    
		CamWriteCmosSensor(0x64, 0x20);                    
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_SEPIAGREEN:                                                 
           CamWriteCmosSensor(0x62, 0x04);                                
		   CamWriteCmosSensor(0x63, 0x50);                    
		   CamWriteCmosSensor(0x64, 0x50);                    
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_SEPIABLUE:                                                  
           CamWriteCmosSensor(0x62, 0x04);                                 
		   CamWriteCmosSensor(0x63, 0x20);                    
		   CamWriteCmosSensor(0x64, 0xf0);                    
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_COLORINV:                                                   
           CamWriteCmosSensor(0x62, 0x01);                                
		   CamWriteCmosSensor(0x63, 0x80);                    
		   CamWriteCmosSensor(0x64, 0x80);                    
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_GRAYINV:                                                    
           CamWriteCmosSensor(0x62, 0x02);                                
		   CamWriteCmosSensor(0x63, 0x80);                    
		   CamWriteCmosSensor(0x64, 0x80);                    
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_EMBOSSMENT:                                                 
           CamWriteCmosSensor(0x62, 0x00);                                
		   CamWriteCmosSensor(0x63, 0x80);                    
		   CamWriteCmosSensor(0x64, 0x80);                    
                                                                                       
           break;                                                                      
                                                                                      
            
        default:
            return KAL_FALSE;
    }
#elif(defined(SP0A18_YUV)||defined(ALL_YUV))
  CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x42);
    switch (In->FeatureSetValue)
    {
       case CAM_EFFECT_ENC_NORMAL:                                                            
            CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0x62, 0x00);
			CamWriteCmosSensor(0x63, 0x80);
			CamWriteCmosSensor(0x64, 0x80);                    
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_GRAYSCALE:                                                  
            CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0x62, 0x20);
			CamWriteCmosSensor(0x63, 0x80);
			CamWriteCmosSensor(0x64, 0x80);                   
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_SEPIA:                                                      
            CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0x62, 0x10);
			CamWriteCmosSensor(0x63, 0xc0);
			CamWriteCmosSensor(0x64, 0x20);                   
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_SEPIAGREEN:                                                 
            CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0x62, 0x10);
			CamWriteCmosSensor(0x63, 0x20);
			CamWriteCmosSensor(0x64, 0x20);                   
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_SEPIABLUE:                                                  
            CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0x62, 0x10);
			CamWriteCmosSensor(0x63, 0x20);
			CamWriteCmosSensor(0x64, 0xf0);                    
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_COLORINV:                                                   
            CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0x62, 0x04);
			CamWriteCmosSensor(0x63, 0x80);
			CamWriteCmosSensor(0x64, 0x80);                   
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_GRAYINV:                                                    
            CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0x62, 0x08);
			CamWriteCmosSensor(0x63, 0x80);
			CamWriteCmosSensor(0x64, 0x80);                    
                                                                                       
           break;                                                                      
                                                                                       
       case CAM_EFFECT_ENC_EMBOSSMENT:                                                 
            CamWriteCmosSensor(0xfd, 0x00);
			CamWriteCmosSensor(0x62, 0x01);
			CamWriteCmosSensor(0x63, 0x80);
			CamWriteCmosSensor(0x64, 0x80);                   
                                                                                       
           break;                                                                      
            
        default:
            return KAL_FALSE;
    }
#endif
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALStillCaptureSize
*
* DESCRIPTION
*   This function get capture size
*
* PARAMETERS
*   In, Out
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
MM_ERROR_CODE_ENUM SP0827_SERIALStillCaptureSize(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  const P_CAL_FEATURE_TYPE_ENUM_STRUCT CapSize = &Out->FeatureInfo.FeatureEnum;
  
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_SUPPORT;
    CapSize->IsSupport = KAL_TRUE;
    CapSize->ItemCount = 3;
    CapSize->SupportItem[0] = CAM_IMAGE_SIZE_WALLPAPER;
	CapSize->SupportItem[1] = CAM_IMAGE_SIZE_QVGA;
    CapSize->SupportItem[2] = CAM_IMAGE_SIZE_VGA;
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALBanding
*
* DESCRIPTION
*   This function get banding setting
*
* PARAMETERS
*   In, Out
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0827_SERIALBanding(P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
#if 1
  const P_CAL_FEATURE_TYPE_ENUM_STRUCT Banding = &Out->FeatureInfo.FeatureEnum;
  
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_VIDEO_SUPPORT;
    Banding->IsSupport = KAL_TRUE;
    Banding->ItemCount = 2;
    Banding->SupportItem[0] = CAM_BANDING_50HZ;
    Banding->SupportItem[1] = CAM_BANDING_60HZ;
  }
  else if (CAL_FEATURE_SET_OPERATION == In->FeatureCtrlCode)
  {
        SP0827_SERIALSensor.BandingFreq = (CAM_BANDING_50HZ == In->FeatureSetValue ? SP0827_SERIAL_50HZ : SP0827_SERIAL_60HZ);
        // TODO: set manual banding here
        {
            if (CAM_BANDING_50HZ == In->FeatureSetValue)
            {
			// based on 24M MCLK 
		
			SP0827_SERIAL_CAM_BANDING_50HZ = KAL_TRUE;
            }
            else
            {			
			SP0827_SERIAL_CAM_BANDING_50HZ = KAL_FALSE;
            }
        }
    }
    return MM_ERROR_NONE;
#endif	
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSceneMode
*
* DESCRIPTION
*   This function get/set scene mode of sensor.
*
* PARAMETERS
*   IsCam: camera scene mode or not
*   In, Out
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static MM_ERROR_CODE_ENUM SP0827_SERIALSceneMode(kal_bool IsCam, P_CAL_FEATURE_CTRL_STRUCT In, P_CAL_FEATURE_CTRL_STRUCT Out)
{
  if (CAL_FEATURE_QUERY_OPERATION == In->FeatureCtrlCode)
  {
    const P_CAL_FEATURE_TYPE_ENUM_STRUCT SceneMode = &Out->FeatureInfo.FeatureEnum;
    
    Out->FeatureType = CAL_FEATURE_TYPE_ENUMERATE;
    Out->FeatureOperationSupport = CAL_FEATURE_SET_OPERATION|CAL_FEATURE_QUERY_OPERATION;
    Out->FeatureModeSupport = CAL_FEATURE_CAMERA_SUPPORT;
    SceneMode->IsSupport = KAL_TRUE;
    SceneMode->ItemCount = 2;
    if (IsCam)
    {
      SceneMode->SupportItem[0] = CAM_AUTO_DSC;
      SceneMode->SupportItem[1] = CAM_NIGHTSCENE;
    }
    else
    {
      SceneMode->SupportItem[0] = CAM_VIDEO_AUTO;
      SceneMode->SupportItem[1] = CAM_VIDEO_NIGHT;
    }
  }
  return MM_ERROR_NONE;
}




#if(defined(SP0829_SERIAL_YUV))
unsigned char Dual_Camera_SP0827_CONFIG[] =
{                                           
	 0x1c , 0x00,                              
	 0x30 , 0x16,                              
	 0x27 , 0x38,                              
	 0x28 , 0x0b,                              
	 0xE7 , 0x03,                              
	 0xE7 , 0x00,                              
	 0x32 , 0x00,                              
	 0x03 , 0x01,                              
	 0x04 , 0x90,                              
	 0x24 , 0x2a, 
	 0x0e , 0x00,
	                                           
	//analog  for 1bit                         
	  0x0f , 0x06,                             
	  0x10 , 0x06,                             
	  0x12 , 0x05,                             
	  0x13 , 0x03,                             
	  0x14 , 0x08,                             
	  0x15 , 0x02,                             
	  0x16 , 0x00,                             
	  0x17 , 0x01,                             
	  0x18 , 0x00,                             
	  0x1a , 0x0f,                             
	                                           
	//AE  24M 8Ƶ for 1bit 15-8fps           
	//ae setting                                                            
	  0x06 , 0x0 ,                             
	  0x09 , 0x0 ,                             
	  0x0a , 0x2c,                             
	//base                                     
	  0x97 , 0x32,                             
	  0x98 , 0x0 ,                             
	//exp                                      
	  0x70 , 0x58,                             
	  0x71 , 0x2 ,                             
	  0x72 , 0x32,                             
	  0x73 , 0x0 ,                             
	  0x78 , 0x32,                             
	  0x79 , 0x0 ,                             
	  0x7a , 0x1 ,                             
	  0x7b , 0x0 ,                             
	//gain                                     
	  0x74 , 0x70,                             
	  0x75 , 0x2a,                             
	  0x7c , 0x50,                             
	  0x7d , 0x2a,                             
	  0x80 , 0x50,                             
	  0x81 , 0x3e,                             
	  0x82 , 0x36,                             
	  0x83 , 0x30,                             
	  0x84 , 0x30,                             
	  0x85 , 0x2e,                             
	  0x86 , 0x2e,                             
	  0x87 , 0x2c,                             
	  0x88 , 0x2c,                             
	  0x89 , 0x2c,                             
	  0x8a , 0x2a,                             
	  0x8b , 0x2a,                             
	  0x8c , 0x2a,                             
	//target                                   
	  0x90 , 0x50, //indoor hold               
	  0x93 , 0x4c,	//idoor lock               
	  0x94 , 0x44,                             
	  0x8f , 0x40,                             
	  0x92 , 0x50,	//otdoor hold              
	  0x95 , 0x4c,	//otdoor lock              
	  0x96 , 0x44,                             
	  0x91 , 0x40,                             
	                                           
	//Status                                                                
	  0x60 , 0x40,//mean_nr_dummy              
	  0x61 , 0x50,//mean_dummy_nr              
	  0x62 , 0x20,//mean_dummy_low             
	  0x63 , 0x35,//mean_low_dummy             
	  0x64 , 0x70,//rpc_heq_low                
	  0x65 , 0x70,//rpc_heq_dummy              
	  0x66 , 0x58,//exp_heq_dummy_8lsm         
	  0x67 , 0x02,//exp_heq_dummy_5hsm         
	  0x68 , 0x58,//exp_heq_low_8lsm           
	  0x69 , 0x02,//exp_heq_low_5hsm           
	                                           
	  0x99 , 0x09,                             
	  0x32 , 0x05,                             
	                                           
 };

unsigned char Dual_Camera_SP0829_CONFIG[] =
{
	0x1c , 0x00,                         
	0x2e , 0xf0,//   for sp0827          
	0x32 , 0x00,                         
	0x2c , 0x05,                         
	0x31 , 0x10, 
	0x30 , 0x06,
	0x35 , 0x40,//0x40
	0x0e , 0xf7,//disable sp0829 analog output
	                                  
	                                  
	//AWB                                  
	0xc5 , 0xc4,                         
	0xc6 , 0x9f,                         
	0xE7 , 0x03,                         
	0xE7 , 0x00,                         
	0xb3 , 0x7e, 
	0xb6 , 0x09,
	0xb7 , 0x09,                        
	0xba , 0x2d,                         
	0xd7 , 0x90,                         
	0xd8 , 0xa0,                         
	0xd9 , 0xff,                         
	0xdd , 0xa6,                         
	0xe1 , 0xc7,                         
	0xe3 , 0xd0,                         
	0xd5 , 0xe0,                         
	                                  
	//smooth                               
	0x42 , 0x54,//03//02//               
	0x46 , 0x0a,                         
	                                  
	//sharpen                              
	0x50 , 0x40,                         
	0x51 , 0x10,                         
	0x52 , 0xe0, 
	                                              
	0x56 , 0x40,                         
	0x57 , 0x20,                         
	//cm                                   
	0x79 , 0xca,                         
	0x7a , 0xb0,                         
	0x7b , 0x6 ,                         
	0x7c , 0xd7,                         
	0x7d , 0xc7,                         
	0x7e , 0xe2,                         
	0x7f , 0xff,                         
	0x80 , 0x96,                         
	0x81 , 0xeb,                         
	0x82 , 0xc ,                         
	0x83 , 0x33,                         
	0x84 , 0xf ,  
	
	/*/color ԭԺ
        0x79,0x96,
        0x7a,0xf1,
        0x7b,0xfa,
        0x7c,0xe5,
        0x7d,0xc6,
        0x7e,0xd6,
        0x7f,0xff,
        0x80,0x96,
        0x81,0xeb,
        0x82,0x3c,
        0x83,0x33,
        0x84,0x0f,*/     
                                       
	//gama
        0x65,0x0 ,
        0x66,0x7 ,
        0x67,0xe ,
        0x68,0x1b,
        0x69,0x38,
        0x6a,0x53,
        0x6b,0x6e,
        0x6c,0x85,
        0x6d,0x97,
        0x6e,0xa6,
        0x6f,0xb6,
        0x70,0xc2,
        0x71,0xce,
        0x72,0xd7,
        0x73,0xe2,
        0x74,0xe9,
        0x75,0xf2,
        0x76,0xf9,
        0x77,0xff,
                                                 
	                                  
	0x33 , 0x0f,                         
	0x34 , 0x06,                         
	0x32 , 0x08,                         

};

#elif(defined(SP0A18_YUV)||defined(ALL_YUV))
unsigned char Dual_Camera_SP0827_CONFIG[] =
{ 
  0x1c , 0x00,                              
  0x30 , 0x16,                              
  0x27 , 0x38,                              
  0x28 , 0x0b,                              
  0xE7 , 0x03,                              
  0xE7 , 0x00,                              
  0x32 , 0x00,                              
  0x03 , 0x01,                              
  0x04 , 0x90,                              
  0x24 , 0x2a, 
  0x0e , 0x00,
  
  0x0f , 0x06,
  0x10 , 0x06, 
  0x12 , 0x05,
  0x13 , 0x03,
  0x14 , 0x08,
  0x15 , 0x02, 
  0x16 , 0x00,
  0x17 , 0x01, 
  0x18 , 0x00,
  0x1a , 0x0f,
         
	//AE  24M 8Ƶ for 1bit 15-8fps           
////ae setting
  0x06 , 0x0 ,
  0x09 , 0x0 ,
  0x0a , 0x2c,                             
////base     
  0x97 , 0x32,
  0x98 , 0x0 ,
////exp     
  0x70 , 0x58,
  0x71 , 0x2 ,
  0x72 , 0x32,
  0x73 , 0x0 ,
  0x78 , 0x32,
  0x79 , 0x0 ,
  0x7a , 0x1 ,
  0x7b , 0x0 ,
//gain     
  0x74 , 0x70,
  0x75 , 0x2a,
  0x7c , 0x50,
  0x7d , 0x2a,
  0x80 , 0x50,
  0x81 , 0x3e,
  0x82 , 0x36,
  0x83 , 0x30,
  0x84 , 0x30,
  0x85 , 0x2e,
  0x86 , 0x2e,
  0x87 , 0x2c,
  0x88 , 0x2c,
  0x89 , 0x2c,
  0x8a , 0x2a,
  0x8b , 0x2a,
  0x8c , 0x2a,
//target     
  0x90 , 0x40,//60 //indoor hold
  0x93 , 0x3c,//5c	//indoor lock
  0x94 , 0x34,//54
  0x8f , 0x30,//50 
  0x92 , 0x40,//60	//outdoor hold
  0x95 , 0x3c,//5c	//outdoor lock
  0x96 , 0x34,//54
  0x91 , 0x30,//50      
            
//status     
        
  0x60 , 0x40,//mean_nr_dummy
  0x61 , 0x50,//mean_dummy_nr
  0x62 , 0x20,//mean_dummy_low
  0x63 , 0x35,//mean_low_dummy
  0x64 , 0x70,//rpc_heq_low
  0x65 , 0x70,//rpc_heq_dummy
  0x66 , 0x58,//exp_heq_dummy_8lsm
  0x67 , 0x02,//exp_heq_dummy_5hsm
  0x68 , 0x58,//exp_heq_low_8lsm
  0x69 , 0x02,//exp_heq_low_5hsm
  
  0x99 , 0x09,         
  0x32 , 0x05,
  0x31 , 0x30,

};

unsigned char Dual_Camera_SP0A18_CONFIG[] = 
{ 
  0xfd , 0x00,
  0x0e , 0xf7, //disable sp0a18 analog output
  0x2e , 0xf0,
  0x2c , 0x0e,
  0x1b , 0x05,
  0x1c , 0x00,//
  
  //240*320 for sp0827
  0xfd , 0x00,
  0x47 , 0x00,
  0x48 , 0x00,
  0x49 , 0x01,
  0x4a , 0x40,
  0x4b , 0x00,
  0x4c , 0x00,
  0x4d , 0x00,
  0x4e , 0xf0,
            
  0x30 , 0x00,
  
  0x32 , 0x00, 
  0xfd , 0x01,
  0x28 , 0xc4,
  0x29 , 0x9e,
  0xfd , 0x00,
  0xe7 , 0x03,
  0xe7 , 0x00,
        
//special   
  0xfd , 0x01,
  0x00 , 0x01,
//pregain   
  0xc6 , 0x0a,//a~f
  0xc7 , 0x0a,//a~f
  0xc8 , 0x0a,//a~f
  0xc9 , 0x0a,//a~f
  
//AWB     
  0xfd , 0x01,
        
  0x10 , 0x0d,//
  0x11 , 0x08,//d
  0x12 , 0x08,//d
  0x17 , 0x27,//2f 
  0x18 , 0x45, 
  0x1a , 0xb2,//AC?//9e
  0x1b , 0xbb,//9e,
  0x2a , 0xE0,
  0x2b , 0x10,//35
  0x2c , 0x9f,//90
  0x2d , 0x99,
  0x2e , 0x99,
  0x6f , 0x0a,//05
  0x8b , 0x2f,
       
  0x20 , 0xd0,
  0x21 , 0x78,
  0x22 , 0xe0,
  0x23 , 0x80,
  0x16 , 0x08,//14
//smooth   
  0xfd , 0x00,
  0x4f , 0x43,//43 raw_gf_fac_outdoor raw_gf_fac_nr 55
  0x50 , 0x11,//11 draw_gf_fac_dummy raw_gf_fac_low
  0x57 , 0x06,//06 raw_dif_thr_outdoor
  0x58 , 0x0f,//16 raw_dif_thr_normal
  0x56 , 0x18,//18 dummy
  0x59 , 0x18,//18 low
          
  0x5a , 0x70,// raw_rb_fac_outdoor,raw_rb_fac_nr
  0x5b , 0x00,// raw_rb_fac_dummy,raw_rb_fac_low
       
//sharpen
  0xfd , 0x01,
  0xe5 , 0x20,
  0xe6 , 0x10,
  0xe7 , 0x20,    
  0xe8 , 0x28,//sharp_fac_pos_outdoor
  0xec , 0x28,//sharp_fac_neg_outdoor
  0xe9 , 0x28,//sharp_fac_pos_nr
  0xed , 0x28,//sharp_fac_neg_nr
  0xea , 0x28,//sharp_fac_pos_dummy
  0xef , 0x28,//sharp_fac_neg_dummy
  0xeb , 0x10,//sharp_fac_pos_low
  0xf0 , 0x20,//sharp_fac_neg_low
        
//gamma     
 0xfd , 0x00,
 0x8b , 0x0 , //,0 
 0x8c , 0xC ,//,11
 0x8d , 0x19,//,19 
 0x8e , 0x2C,//,28 
 0x8f , 0x49,//,46 
 0x90 , 0x61,//,61 
 0x91 , 0x77,//,78 
 0x92 , 0x8A,//,8A 
 0x93 , 0x9B,//,9B 
 0x94 , 0xA9,//,A9 
 0x95 , 0xB5,//,B5 
 0x96 , 0xC0,//,C0 
 0x97 , 0xCA,//,CA 
 0x98 , 0xD4,//,D4 
 0x99 , 0xDD,//,DD 
 0x9a , 0xE6,//,E6 
 0x9b , 0xEF,//,EF 
 0xfd , 0x01,//,01 
 0x8d , 0xF7,//,F7 
 0x8e , 0xFF,//,FF 
             
//uv en     
  0xfd , 0x00,
  0xb0 , 0x00,//uv_dif_gain_outdoor,uv_dif_gain_nr
  0xb1 , 0x33,//uv_dif_gain_dummy,uv_dif_gain_low   
        
//neg       
  0xfd , 0x01,
  0xd2 , 0x00, 
      
//lowlight lu
  0xfd , 0x00,
  0xb2 , 0x08,//lum_limit
  0xb3 , 0x1f,//lum_set
  0xb4 , 0x20,//black_vt
  0xb5 , 0x45,//white_vt
      
//auto sat   
  0xfd , 0x00,
  0xbf , 0x01,//autosa_en
  0xc0 , 0xff,//a0//y_p_th
  0xc1 , 0xff,//a0//y_mean_th
           
//saturation 
  0xfd , 0x00,
  0xd8 , 0x80,//74//88sat_uv_s1
  0xd9 , 0x80,//74//88sat_uv_s2
  0xda , 0x80,//74//88sat_uv_s3
  0xdb , 0x80,//74//88sat_uv_s4
        
//heq        
  0xfd , 0x00,
  0xdc , 0x00,//heq_offset
  0xdd , 0x70,///ku
  0xde , 0xb0,//b0//kl
  0xdf , 0x70,//heq_mean
          
//CCM        
  0xfd , 0x01,
  0xa0 , 0x80,//80(ɫӽɫ)
  0xa1 , 0x0 ,//0 
  0xa2 , 0x0 ,//0 
  0xa3 , 0xf3,//f0
  0xa4 , 0x8e,//a6
  0xa5 , 0x0 ,//ea
  0xa6 , 0x0 ,//0 
  0xa7 , 0xe6,//e6
  0xa8 , 0x9a,//9a
  0xa9 , 0x0 ,//0 
  0xaa , 0x3 ,//33
  0xab , 0xc ,//c 
     
//skin      
  0xfd , 0x01,
           
  0x8b , 0x2f,//bit7,0 en
  0x01 , 0xf0,
  0x02 , 0x04,
  0x03 , 0x70,
  0x04 , 0x99,//88
  0x05 , 0x24,//13
//resize_en  
  0xfd , 0x01,
  0x0e , 0x00,
         
//outformat  
  0xfd , 0x00,
  0x35 , 0x80,
  0x5f , 0x04,//01
         
//En AWB     
  0xfd , 0x00,
  0x31 , 0x10,
  0x32 , 0x08,
  0x33 , 0x0f,  
  0x34 , 0x06, 

};

#endif

/*************************************************************************
* FUNCTION
*   SP0827_SERIALInitialSetting
*
* DESCRIPTION
*   This function initialize the registers of CMOS sensor
*
* PARAMETERS
*   None
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
extern kal_uint16 firstentersp0a18;
static void SP0827_SERIALInitialSetting(void)
{		
	int i,reg_count;
	CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x80);
	reg_count = sizeof(Dual_Camera_SP0827_CONFIG) / 2;
	dbg_print("reg_count=%d\r\n",reg_count);
	
	dbg_print("SP0827:\r\n");
	for (i = 0;i < reg_count;i++)
	{
		CamWriteCmosSensor(Dual_Camera_SP0827_CONFIG[i*2],Dual_Camera_SP0827_CONFIG[i*2+1] );
		dbg_print("%x value=%x\r\n",Dual_Camera_SP0827_CONFIG[i*2],CamReadCmosSensor(Dual_Camera_SP0827_CONFIG[i*2]));
	}

	#if(defined(SP0829_SERIAL_YUV))
	dbg_print("SP0829:\r\n");
	CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x86);
	reg_count = sizeof(Dual_Camera_SP0827_CONFIG) / 2;
	for (i = 0;i < reg_count;i++)
	{
		
		CamWriteCmosSensor(Dual_Camera_SP0829_CONFIG[i*2],Dual_Camera_SP0829_CONFIG[i*2+1] );
		dbg_print("%x value=%x\r\n",Dual_Camera_SP0829_CONFIG[i*2],CamReadCmosSensor(Dual_Camera_SP0829_CONFIG[i*2]));
	}

	#elif(defined(SP0A18_YUV)||defined(ALL_YUV))
	CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x42);
	
	dbg_print("SP0A18:\r\n");
	reg_count = sizeof(Dual_Camera_SP0A18_CONFIG) / 2;
	for (i = 0;i < reg_count;i++)
	{
		
		CamWriteCmosSensor(Dual_Camera_SP0A18_CONFIG[i*2],Dual_Camera_SP0A18_CONFIG[i*2+1] );
		dbg_print("%x value=%x\r\n",Dual_Camera_SP0A18_CONFIG[i*2],CamReadCmosSensor(Dual_Camera_SP0A18_CONFIG[i*2]));
	}

	#endif
	firstentersp0a18=1;
}

/*
static void SP0827_SERIALInitialSetting(void)
{	
}
*/

/*************************************************************************
* FUNCTION
*   SP0827_SERIALPvSetting
*
* DESCRIPTION
*   This function apply the preview mode setting, normal the preview size is 1/4 of full size.
*   Ex. 2M (1600 x 1200)
*   Preview: 800 x 600 (use sub-sample or binning to acheive it)
*   Full Size: 1600 x 1200 (output every effective pixels.)
*
* PARAMETERS
*   None
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALPvSetting(void)
{
    // TODO: add preview setting here
    
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALCapSetting
*
* DESCRIPTION
*   This function config capture settting to sensor
*
* PARAMETERS
*   None
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALCapSetting(void)
{
	SP0827_SERIALSensor.Pclk = 24000000;		/* It is the PCLK for sensor internal parallel interface */	
	SP0827_SERIALSensor.PvMode = KAL_FALSE;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALGetSensorInfo
*
* DESCRIPTION
*   This function set sensor infomation
*
* PARAMETERS
*   Info
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALGetSensorInfo(P_IMAGE_SENSOR_GET_SENSOR_INFO_STRUCT Info)
{
	Info->SensorId = SP0827_SERIAL_SENSOR_ID;
	Info->SensorIf = IMAGE_SENSOR_IF_SERIAL;

	Info->PreviewMclkFreq = SP0827_SERIAL_MCLK;
	Info->CaptureMclkFreq = SP0827_SERIAL_MCLK;
	Info->VideoMclkFreq = SP0827_SERIAL_MCLK;

	Info->PreviewWidth = SP0827_SERIAL_IMAGE_SENSOR_PV_WIDTH;
	Info->PreviewHeight = SP0827_SERIAL_IMAGE_SENSOR_PV_HEIGHT;
	Info->FullWidth = SP0827_SERIAL_IMAGE_SENSOR_FULL_WIDTH;
	Info->FullHeight = SP0827_SERIAL_IMAGE_SENSOR_FULL_HEIGHT;

	/* data format */
	Info->PreviewNormalDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_YUYV;
	Info->PreviewHMirrorDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_YUYV;
	Info->PreviewVMirrorDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_YUYV;
	Info->PreviewHVMirrorDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_YUYV;
	Info->CaptureDataFormat = IMAGE_SENSOR_DATA_OUT_FORMAT_YUYV;

	/* isp pclk invert switch */
	Info->PixelClkInv 			= KAL_FALSE;
	Info->PixelClkPolarity 		= POLARITY_LOW;

	/* HSYNC/VSYNC polarity */
	Info->HsyncPolarity = POLARITY_LOW; /* Actually high active when set POLARITY_LOW */
	Info->VsyncPolarity = POLARITY_LOW;

	Info->SensorAfSupport = KAL_FALSE;
	Info->SensorFlashSupport = KAL_FALSE;

	/* If IsSensorDriverCtrlMCLK is TRUE, then the MCLK turn on/off is controlled by sensor driver.*/
	Info->IsSensorDriverCtrlMclk = KAL_TRUE;
}

/*************************************************************************
* FUNCTION
*    SP0827SerialSensorFeatureCtrl
*
* DESCRIPTION
*    This function set sensor feature mode
*
* PARAMETERS
*    id: scenario id
*
* RETURNS
*    error code
*
* LOCAL AFFECTED
*
*************************************************************************/
MM_ERROR_CODE_ENUM SP0827_SERIALGetSerialSensorInfo(P_SERIAL_SENSOR_INFO_IN_STRUCT In,P_SERIAL_SENSOR_INFO_OUT_STRUCT Out)
{
	if (In->ScenarioId == CAL_SCENARIO_CAMERA_STILL_CAPTURE)
	{
		Out->para_1 = KAL_TRUE; 								/* SCK clock inverse */
		Out->para_2 = 4;										/* Cycle */
		Out->para_3 = SP0827_SERIAL_IMAGE_SENSOR_PV_WIDTH; 	/* Sensor capture data valid width */
		Out->para_4 = SP0827_SERIAL_IMAGE_SENSOR_PV_HEIGHT;	/* Sensor capture data valid height */
	}
	else
	{
		Out->para_1 = KAL_TRUE; 								/* SCK clock inverse */
		Out->para_2 = 4;										/* Cycle */
		Out->para_3 = SP0827_SERIAL_IMAGE_SENSOR_FULL_WIDTH; 	/* Sensor preview data valid width */
		Out->para_4 = SP0827_SERIAL_IMAGE_SENSOR_FULL_HEIGHT; 	/* Sensor preview data valid height */
	}

	return MM_ERROR_NONE; 
}




/*************************************************************************
* FUNCTION
*   SP0827_SERIALPowerOn
*
* DESCRIPTION
*   This function apply the power on sequence and read the sensor ID to check if the I2C
*  communication can work or not.
*
* PARAMETERS
*   None
*
* RETURNS
*   Sensor id
*
* LOCAL AFFECTED
*
*************************************************************************/
static kal_uint32 SP0827_SERIALPowerOn(void)
{

  kal_uint32 SensorId = 0;
  
  CisModulePowerOn(SP0827_SERIALSensor.SensorIdx, KAL_TRUE);
  CameraSccbOpen(CAMERA_SCCB_SENSOR, SP0827_SERIAL_WRITE_ID_0,SP0827_SERIAL_I2C_ADDR_BITS, SP0827_SERIAL_I2C_DATA_BITS, SP0827_SERIAL_HW_I2C_SPEED);
  SP0827_SERIALSensor.sccb_opened = KAL_TRUE;
  kal_sleep_task(3);

  /* Enable the MCLK */
  ENABLE_CAMERA_CLOCK_OUTPUT_TO_CMOS;
  kal_sleep_task(2);
  
  CamPdnPinCtrl(0xFF, 1);
  kal_sleep_task(4);

#if defined(DUAL_CAMERA_SUPPORT)
	   SP0827_SERIALSensor.SensorIdx=camera_front_or_back;
#endif


   /* set sensors chip enable pin to low to activate SP0827 */
   //CamPdnPinCtrl(SP0827_SERIALSensor.SensorIdx, 0);   //need to check power on sequency //
  	//CamPdnPinCtrl(0, 0);
  	CamPdnPinCtrl(1, 0);
	GPIO_ModeSetup(43, 0);
	GPIO_InitIO(1, 43);
	GPIO_WriteIO(0, 43);

  
   kal_sleep_task(1);
   
  /* Reset the sensor */
  CamRstPinCtrl(SP0827_SERIALSensor.SensorIdx, 1);
  kal_sleep_task(1);
  CamRstPinCtrl(SP0827_SERIALSensor.SensorIdx, 0);
  kal_sleep_task(4);
  CamRstPinCtrl(SP0827_SERIALSensor.SensorIdx, 1);
  kal_sleep_task(2);
  
   CameraSccbSetAddr(CAMERA_SCCB_SENSOR, SP0827_SERIAL_WRITE_ID_0);
   kal_sleep_task(5);
    	
/*******gpio CMPDN*****/
      GPIO_ModeSetup(SP0827_SERIAL_POWERDOWN_PIN, 0);
      GPIO_InitIO(1, SP0827_SERIAL_POWERDOWN_PIN);
   
      kal_sleep_task(4);
      GPIO_WriteIO(0, SP0827_SERIAL_POWERDOWN_PIN);
      kal_sleep_task(1); 
      GPIO_WriteIO(1, SP0827_SERIAL_POWERDOWN_PIN);
      kal_sleep_task(4); 
      GPIO_WriteIO(0, SP0827_SERIAL_POWERDOWN_PIN);
      kal_sleep_task(2); 
	  
	CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x80);
   SensorId = CamReadCmosSensor(0x02);
	   
    #ifndef WIN32
       dbg_print("SP0827_SERIALPowerOn  SensorId=%x\r\n", SensorId);
       kal_prompt_trace(MOD_ENG,"SP0827_SERIALPowerOn SP0827SensorId = %x\r\n",SensorId);
    #endif	       
       zh_print("SP0827_SERIALPowerOn  SensorId=%x\r\n", SensorId);

  return SensorId;
}
/*************************************************************************
* FUNCTION
*   SP0827_SERIALPreview
*
* DESCRIPTION
*   This function is the most important functions of (Init, Preview & Capture). 
*   1. Apply preview setting, make sensor output preview resolution, e.g. 800x600 for 2M sensor.
*   2. Record video resolution, and set mode based on the isp operation mode.
*   3. Configure the grab window.
*
* PARAMETERS
*   Id, In, Out
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALPreview(CAL_SCENARIO_ENUM Id, P_IMAGE_SENSOR_SCENARIO_PARA_IN_STRUCT In, P_IMAGE_SENSOR_SCENARIO_PARA_OUT_STRUCT Out)
{
       //zh_print("...... (%s,%d) \r\n",__func__,__LINE__);

	if(0== In->BandingFreq)
	{
		SP0827_SERIAL_CAM_BANDING_50HZ =CAM_BANDING_50HZ;
	}
	else if(1== In->BandingFreq)
	{
		SP0827_SERIAL_CAM_BANDING_50HZ = CAM_BANDING_60HZ;
	}
	SP0827_SERIAL_CAM_nightmode = In->NightMode;
	SP0827_SERIALPvSetting();

	switch (Id)
	{
	case CAL_SCENARIO_VIDEO:
		video_mode = KAL_TRUE;
		Out->WaitStableFrameNum = 3;
		break;
	default:

		video_mode = KAL_FALSE;
		Out->WaitStableFrameNum = 3;
		break;
	}
  
	//SP0827_SERIALSetMirror(In->ImageMirror);

	SP0827_SERIALAeEnable(KAL_TRUE);
	SP0827_SERIALAwbEnable(KAL_TRUE);

	SP0827_SERIALNightMode(video_mode,SP0827_SERIAL_CAM_BANDING_50HZ,SP0827_SERIAL_CAM_nightmode);

	Out->GrabStartX = SP0827_SERIAL_PV_GRAB_START_X;
	Out->GrabStartY = SP0827_SERIAL_PV_GRAB_START_Y;
	Out->ExposureWindowWidth = SP0827_SERIAL_PV_GRAB_WIDTH;
	Out->ExposureWindowHeight = SP0827_SERIAL_PV_GRAB_HEIGHT;
       zh_print("...... (%s,%d) \r\n",__func__,__LINE__);
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALCapture
*
* DESCRIPTION
*   This function is the most important functions of (Init, Preview & Capture). 
*   1. Apply capture setting when capture size greatter then preview size, make sensor output 
*     full size, e.g. 1600x1200 for 2M sensor.
*   2. Increase the dummy pixels or derease pixel clock based on the digital zoom factor and
*    interpolation times.
*   3. Re-Calculate the shutter and sensor gain, to make the exposure value is same with preview
*   4. Configure the grab window based on sensor output.
*
* PARAMETERS
*   Id, In, Out
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALCapture(CAL_SCENARIO_ENUM Id, P_IMAGE_SENSOR_SCENARIO_PARA_IN_STRUCT In, P_IMAGE_SENSOR_SCENARIO_PARA_OUT_STRUCT Out)
{
	SP0827_SERIALCapSetting();
	
	Out->WaitStableFrameNum = SP0827_SERIAL_CAPTURE_DELAY_FRAME;

	Out->GrabStartX = SP0827_SERIAL_FULL_GRAB_START_X;
	Out->GrabStartY = SP0827_SERIAL_FULL_GRAB_START_Y;
	Out->ExposureWindowWidth = SP0827_SERIAL_FULL_GRAB_WIDTH;
	Out->ExposureWindowHeight = SP0827_SERIAL_FULL_GRAB_HEIGHT;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALDetectSensorId
*
* DESCRIPTION
*   This function detect sensor id
*
* PARAMETERS
*   None
*
* RETURNS
*   Sensor id
*
* LOCAL AFFECTED
*
*************************************************************************/
static kal_uint32 SP0827_SERIALDetectSensorId(void)
{
   //kal_uint16 SP0827_SERIAL_sensor_id = 0;
    MM_ERROR_CODE_ENUM SP0827_SERIALSensorClose(void);
    kal_uint32 SensorId;
  	IMAGE_SENSOR_INDEX_ENUM AnotherSensorIdx;
	kal_int8 i, j;

	if (IMAGE_SENSOR_MAIN == SP0827_SERIALSensor.SensorIdx)
	{
		AnotherSensorIdx = IMAGE_SENSOR_SUB;
	}
	else
	{
		AnotherSensorIdx = IMAGE_SENSOR_MAIN;
	}
for (i = 1; i >= 0; i--)
{
	for (j = 1; j >= 0; j--)
	{
	CamRstPinCtrl(AnotherSensorIdx, i);
	CamPdnPinCtrl(AnotherSensorIdx, j);
      SensorId = SP0827_SERIALPowerOn();
      SP0827_SERIALSensorClose();
      if (SP0827_SERIAL_SENSOR_ID== SensorId)
      {
        return SP0827_SERIAL_SENSOR_ID;
      }
    }
  }
  return MM_ERROR_SENSOR_READ_ID_FAIL;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALInitOperationPara
*
* DESCRIPTION
*   This function set sensor operation para
*
* PARAMETERS
*   Para
*
* RETURNS
*   None
*
* LOCAL AFFECTED
*
*************************************************************************/
static void SP0827_SERIALInitOperationPara(P_IMAGE_SENSOR_GET_INIT_OPERATION_PARA_STRUCT Para)
{
  Para->CaptureDelayFrame = 0; /* use WaitStableFrameNum instead of this */
  Para->PreviewDelayFrame = SP0827_SERIAL_PREVIEW_DELAY_FRAME;
  Para->PreviewDisplayWaitFrame = SP0827_SERIAL_FIRST_PREVIEW_DELAY_FRAME;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSensorOpen
*
* DESCRIPTION
*   This function read sensor id and init sensor
*
* PARAMETERS
*   None
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/

 MM_ERROR_CODE_ENUM SP0827_SERIALSensorOpen(void)
{
  if (SP0827_SERIALPowerOn() != SP0827_SERIAL_SENSOR_ID)
  {
    return MM_ERROR_SENSOR_READ_ID_FAIL;
  }
  GPIO_ModeSetup(59,2);   	//CMPCLK	
  GPIO_ModeSetup(47,2);   	//Data0

  SP0827_SERIALInitialSetting(); /* apply the sensor initial setting */
  /* default setting */
  SP0827_SERIALSensor.BypassAe = KAL_FALSE;
  SP0827_SERIALSensor.BypassAwb = KAL_FALSE;
  SP0827_SERIALSensor.PvMode = KAL_TRUE;
  SP0827_SERIALSensor.BandingFreq = CAM_BANDING_50HZ;
  SP0827_SERIALSensor.InternalClock = 0;
  SP0827_SERIALSensor.Shutter = 1;
  //SP0827_SERIALSensor.Gain = 0x40;
  SP0827_SERIALSensor.FrameLength = 0;
  SP0827_SERIALSensor.LineLength = 0;
  
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSensorFeatureCtrl
*
* DESCRIPTION
*   This function set sensor feature mode
*
* PARAMETERS
*   Id: scenario id
*   In: input parameter
*   Out: output parameter
*   OutLen: output parameter length
*   RealOutLen: real output parameter length
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
 MM_ERROR_CODE_ENUM SP0827_SERIALSensorFeatureCtrl(kal_uint32 Id, void *In, void *Out, kal_uint32 OutLen, kal_uint32 *RealOutLen)
{
  MM_ERROR_CODE_ENUM ErrCode = MM_ERROR_NONE;
  
  if (Id >= CAL_FEATURE_WEBCAM_BEGIN && Id <= CAL_FEATURE_WEBCAM_END)
  {
    return SP0827_SERIALWebcamFeatureCtrl(Id, In, Out, OutLen, RealOutLen);
  }
  switch (Id)
  {
  /* query and set series */
  case CAL_FEATURE_CAMERA_BRIGHTNESS:
  case CAL_FEATURE_CAMERA_EV_VALUE:
    ErrCode = SP0827_SERIALEv(In, Out);
    break;
  case CAL_FEATURE_CAMERA_CONTRAST:
    ErrCode = SP0827_SERIALContrast(In, Out);
    break;
  case CAL_FEATURE_CAMERA_SATURATION:
    ErrCode = SP0827_SERIALSaturation(In, Out);
    break;
  case CAL_FEATURE_CAMERA_SHARPNESS:
    ErrCode = SP0827_SERIALSharpness(In, Out);
    break;
  case CAL_FEATURE_CAMERA_WB:
    ErrCode = SP0827_SERIALWb(In, Out);
    break;
  case CAL_FEATURE_CAMERA_IMAGE_EFFECT:
    ErrCode = SP0827_SERIALEffect(In, Out);
    break;
  case CAL_FEATURE_CAMERA_STILL_CAPTURE_SIZE:
    ErrCode = SP0827_SERIALStillCaptureSize(In, Out);
    break;
  case CAL_FEATURE_CAMERA_BANDING_FREQ:
    ErrCode = ErrCode = SP0827_SERIALBanding(In, Out);
    break;
  case CAL_FEATURE_CAMERA_SCENE_MODE:
    ErrCode = SP0827_SERIALSceneMode(KAL_TRUE, In, Out);
    break;
  case CAL_FEATURE_CAMERA_VIDEO_SCENE_MODE:
    ErrCode = SP0827_SERIALSceneMode(KAL_FALSE, In, Out);
    break;
  case CAL_FEATURE_CAMERA_FLASH_MODE:
  case CAL_FEATURE_CAMERA_HUE:
  case CAL_FEATURE_CAMERA_GAMMA:
    ErrCode = MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    break;
  
  /* get info series */
  case IMAGE_SENSOR_FEATURE_GET_SENSOR_INFO:
    SP0827_SERIALGetSensorInfo(Out);
    break;
  case IMAGE_SENSOR_FEATURE_GET_SENSOR_ID:
    ((P_IMAGE_SENSOR_COMM_DATA_STRUCT)Out)->FeatureValue = SP0827_SERIALDetectSensorId();
    break;
  case IMAGE_SENSOR_FEATURE_GET_SENSOR_FRAME_RATE:
    ((P_IMAGE_SENSOR_COMM_DATA_STRUCT)Out)->FeatureValue = SP0827_SERIALSensor.MaxFrameRate;
    break;
  case IMAGE_SENSOR_FEATURE_GET_INIT_OPERATION_PARA:
    SP0827_SERIALInitOperationPara(Out);
    break;
  case IMAGE_SENSOR_FEATURE_GET_HW_DEPENDENT_SETTING:
    CamGetHWInfo(SP0827_SERIALSensor.SensorIdx, Out);
    break;
  
  /* Set para series */
  case IMAGE_SENSOR_FEATURE_SET_SENSOR_POWER_DOWN:
   // SP0827_SERIALPowerDown();
    break;
  case IMAGE_SENSOR_FEATURE_SET_NVRAM_SENSOR_INFO:
    SP0827_SERIALSensor.NvramData = &(((nvram_camera_para_struct *)In)->SENSOR);
    break;
  case IMAGE_SENSOR_FEATURE_SET_CAMERA_SOURCE:
    SP0827_SERIALSensor.SensorIdx = ((P_IMAGE_SENSOR_SET_CAMERA_SOURCE_STRUCT)In)->ImageSensorIdx;
    break;
  case IMAGE_SENSOR_FEATURE_CTRL_NIGHTMODE: /* move to preview */
    break;
  case IMAGE_SENSOR_FEATURE_SET_AE_BYPASS:
    SP0827_SERIALSensor.BypassAe = *(kal_bool *)In;
    SP0827_SERIALAeEnable(SP0827_SERIALSensor.BypassAe ? KAL_FALSE : KAL_TRUE);
    break;
  case IMAGE_SENSOR_FEATURE_SET_AWB_BYPASS:
    SP0827_SERIALSensor.BypassAwb = *(kal_bool *)In;
    SP0827_SERIALAwbEnable(SP0827_SERIALSensor.BypassAwb ? KAL_FALSE : KAL_TRUE);
    break;
case IMAGE_SENSOR_FEATURE_GET_SERIAL_SENSOR_INFO:
    SP0827_SERIALGetSerialSensorInfo(In, Out);
     break;
  default:
    ErrCode = MM_ERROR_SENSOR_FEATURE_NOT_SUPPORT;
    break;
  }
  return ErrCode;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSensorCtrl
*
* DESCRIPTION
*   This function set sensor scenario mode
*
* PARAMETERS
*   Id: scenario id
*   In/Out: parameter
*   OutLen: out parameter length
*   RealOutLen: real out parameter length
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
 MM_ERROR_CODE_ENUM SP0827_SERIALSensorCtrl(CAL_SCENARIO_ENUM Id, void *In, void *Out, kal_uint32 OutLen, kal_uint32 *RealOutLen)
{
  switch (Id)
  {
  case CAL_SCENARIO_CAMERA_PREVIEW:
  case CAL_SCENARIO_VIDEO:
  case CAL_SCENARIO_WEBCAM_PREVIEW:
  case CAL_SCENARIO_WEBCAM_CAPTURE:
    SP0827_SERIALPreview(Id, In, Out);
    break;
  case CAL_SCENARIO_CAMERA_STILL_CAPTURE:
    SP0827_SERIALCapture(Id, In, Out);
    break;
  default:
    break;
  }
  return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSensorClose
*
* DESCRIPTION
*   This function is to turn off sensor module power.
*
* PARAMETERS
*   None
*
* RETURNS
*   Error code
*
* LOCAL AFFECTED
*
*************************************************************************/
void SP0827_SERIAL_Disable(void)
{      CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x80); 
	CamWriteCmosSensor(0x0e,0xff);
	CamWriteCmosSensor(0x1c,0x03);	
	CamWriteCmosSensor(0xe7,0x03);
	CamWriteCmosSensor(0xe7,0x00);	
}

 MM_ERROR_CODE_ENUM SP0827_SERIALSensorClose(void)
{
CameraSccbSetAddr(CAMERA_SCCB_SENSOR, 0x80);
	CamWriteCmosSensor(0x0e, 0xff);//sensor sleep mode
	CamWriteCmosSensor(0x1c, 0x03);
	CamWriteCmosSensor(0xe7, 0x03);
	CamWriteCmosSensor(0xe7, 0x00);
   #if defined(DUAL_CAMERA_SUPPORT)
        SP0827_SERIALSensor.SensorIdx=camera_front_or_back;
   #endif

	/* Set Low to MCLK */
	DISABLE_CAMERA_CLOCK_OUTPUT_TO_CMOS;

	CisModulePowerOn(SP0827_SERIALSensor.SensorIdx, KAL_FALSE);
	kal_sleep_task(2);
	CamPdnPinCtrl(0xFF, 0);

	SP0827_SERIALSensor.sccb_opened = KAL_FALSE;
	CameraSccbClose(CAMERA_SCCB_SENSOR);
	GPIO_ModeSetup(59,1);   	//CMPCLK	
	GPIO_ModeSetup(47,1);   	//Data0
	return MM_ERROR_NONE;
}

/*************************************************************************
* FUNCTION
*   SP0827_SERIALSensorFunc
*
* DESCRIPTION
*   This function get sensor function pointer
*
* PARAMETERS
*   Sensor function pointer
*
* RETURNS
*   Error code
*
* GLOBALS AFFECTED
*
*************************************************************************/
MM_ERROR_CODE_ENUM SP0827_SERIALSensorFunc(P_IMAGE_SENSOR_FUNCTION_STRUCT *pfSensorFunc)
{
  static IMAGE_SENSOR_FUNCTION_STRUCT ImageSensorFuncSP0827_SERIAL =
  {
    SP0827_SERIALSensorOpen,
    SP0827_SERIALSensorFeatureCtrl,
    SP0827_SERIALSensorCtrl,
    SP0827_SERIALSensorClose,
  };
  
  *pfSensorFunc = &ImageSensorFuncSP0827_SERIAL;
  
  return MM_ERROR_NONE;
}



