/******************************************************************************
 ** File Name:    Sensor_SP0A18.c                                         *
 ** Author:        superpix                                      	  *
 ** Date:           03/02/2011                                          *
 ** Copyright:    Spreadtrum All Rights Reserved.        *
 ** Description:   implementation of digital camera register interface       *
 ******************************************************************************

 ******************************************************************************
 **                        Edit History                                       *
 ** ------------------------------------------------------------------------- *
 ** DATE           NAME             DESCRIPTION                               *
 **                                  *
 ******************************************************************************/

#ifndef _SP0A18_C_
#define _SP0A18_C_

#include "sensor_cfg.h"
#include "sensor_drv.h"
#include "i2c_drv.h"
#include "os_api.h"
#include "chip.h"
#include "dal_dcamera.h"

/**---------------------------------------------------------------------------*
 **                         Const variables                                   *
 **---------------------------------------------------------------------------*/
#define SP0A18_I2C_ADDR_W    			0x42
#define SP0A18_I2C_ADDR_R    			0x43
#define SP0A18_I2C_ACK				       0x0
LOCAL uint32			s_sensor_mclk		 = 0;
LOCAL uint32			Antiflicker		 = DCAMERA_FLICKER_50HZ;
LOCAL BOOLEAN g_isMain = TRUE;
#define VDOREC_20_20FPS        0//video record daylight
#define VDOREC_20_20FPS_MAXGAIN                    0x8c
#define VDOREC_18_18FPS        0//video record daylight
#define VDOREC_18_18FPS_MAXGAIN                    0x88
#define VDOREC_16_16FPS        1//video record daylight
#define VDOREC_16_16FPS_MAXGAIN                   0x84
#define VDOREC_14_14FPS        0//video record daylight
#define VDOREC_14_14FPS_MAXGAIN                   0x80 
#define VDOREC_12_12FPS        0//video record daylight
#define VDOREC_12_12FPS_MAXGAIN                   0X7c

#define CAPPRE_20_10FPS         0//capture preview daylight
#define CAPPRE_20_10FPS_MAXGAIN                  0x78  
#define CAPPRE_20_09FPS         0//capture preview daylight
#define CAPPRE_20_09FPS_MAXGAIN                 0x74 
#define CAPPRE_20_08FPS         1//capture preview daylight
#define CAPPRE_20_08FPS_MAXGAIN                 0x70
#define CAPPRE_20_07FPS         0//capture preview daylight
#define CAPPRE_20_07FPS_MAXGAIN                 0x6c
#define CAPPRE_20_06FPS         0//capture preview daylight  
#define CAPPRE_20_06FPS_MAXGAIN                 0x68 
#define CAPPRE_20_06FPS_NIGHT         1//capture preview nightmode
#define CAPPRE_20_06FPS_MAXGAIN_NIGHT     0Xa0 

#define NORMAL_Y0ffset  0x08
#define LOWLIGHT_Y0ffset  0x25

//AE target
#define  Pre_Value_P0_0xf7  0x88
#define  Pre_Value_P0_0xf8  0x80
#define  Pre_Value_P0_0xf9  0x70
#define  Pre_Value_P0_0xfa  0x68
//HEQ
#define  Pre_Value_P0_0xdd  0x80
#define  Pre_Value_P0_0xde  0xa8
//AWB pre gain
#define  Pre_Value_P1_0x28  0xc4
#define  Pre_Value_P1_0x29  0x9e

/**---------------------------------------------------------------------------*
 **                         Macro define                                      *
 **---------------------------------------------------------------------------*/
#define    SP0A18_DELAY_MS(ms)            {\
                                              uint32 end = SCI_GetTickCount() + ms;\
                                              while (SCI_GetTickCount() < end)\
                                              ;\
                                          }

/**---------------------------------------------------------------------------*
     IF SUPPORT SUB CAMERA WITH SP0827 DEFINE THIS
 **---------------------------------------------------------------------------*/
#define SUB_CAMERA_SUPPORT


/**---------------------------------------------------------------------------*
 **                     Local Function Prototypes                             *
 **---------------------------------------------------------------------------*/
LOCAL uint32 SP0A18_Power_On(uint32 power_on);


#ifdef DOUBLE_CAMERA_SUPPORT
__align(4) SENSOR_REG_T SP0A18_YVU_640X480[] = 
#else
__align(4) const SENSOR_REG_T SP0A18_YVU_640X480[] = 
#endif
{  

//SP0A18 ini
{0xfd , 0x00},
{0x0e , 0x00},
{0x1C , 0x00},
{0x1b , 0x2f},
{0x32 , 0x00},
{0xfd , 0x01},
{0x28 , Pre_Value_P1_0x28},
{0x29 , Pre_Value_P1_0x29},  
{0xfd , 0x00},
{0xe7 , 0x03},
{0xe7 , 0x00},
{0x0f , 0x3f},
{0x10 , 0x3a},
{0x13 , 0x1f},
{0x15 , 0x0a},
{0x16 , 0x00},
{0x1a , 0x09},
{0x44 , 0x0f},
{0x22 , 0x13},
{0x26 , 0x02},
{0x27 , 0x68},
{0x28 , 0x12},
{0x30 , 0x00}, 
{0x31 , 0x10},
{0x34 , 0x00}, 
//special
{0xfd , 0x01},
{0x00 , 0x01},
//pregain
{0xfd , 0x01},
{0xc6 , 0x0a},//a-f
{0xc7 , 0x0a},//a-f
{0xc8 , 0x0a},//a-f
{0xc9 , 0x0a},//a-f
//Del sunspot
{0xfd , 0x00},
{0xcd , 0x24},
{0xd1 , 0x03},
{0xd5 , 0xff},
{0xfc , 0x03},
{0xfb , 0x0f},
//Blacklevel
{0xfd , 0x00},
{0x65 , 0x00},//blue_suboffset
{0x66 , 0x00},//red_suboffset
{0x67 , 0x00},//gr_suboffset
{0x68 , 0x00},//gb_suboffset
//AWB
{0xfd , 0x01},
{0x10 , 0x0d},
{0x11 , 0x0d},
{0x12 , 0x0d},
{0x17 , 0x27}, 
{0x18 , 0x45},  
{0x1a , 0xac},
{0x1b , 0x9e},
{0x2a , 0xE0},
{0x2b , 0x10}, 
{0x2c , 0x9f},
{0x2d , 0x99},
{0x2e , 0x99},
{0x6f , 0x05},
{0x8b , 0x2f},
{0x20 , 0xd0},
{0x21 , 0x78},
{0x22 , 0xe0},
{0x23 , 0x80},
{0x16 , 0x0d},
//smooth
#if 1 //Ԥ
{0xfd , 0x00},
{0x4f , 0x54},//raw_gf_fac_outdoor raw_gf_fac_nr 
{0x50 , 0x22},//draw_gf_fac_dummy raw_gf_fac_low
{0x57 , 0x06},//raw_dif_thr_outdoor
{0x58 , 0x0f},//raw_dif_thr_normal
{0x56 , 0x10},//dummy
{0x59 , 0x10},//low
{0x5a , 0x70},//raw_rb_fac_outdoor{raw_rb_fac_nr
{0x5b , 0x00},//raw_rb_fac_dummy{raw_rb_fac_low
#else //ԤԽ
{0xfd , 0x00},
{0x4f , 0x43},//raw_gf_fac_outdoor raw_gf_fac_nr 
{0x50 , 0x11},//draw_gf_fac_dummy raw_gf_fac_low
{0x57 , 0x06},//raw_dif_thr_outdoor
{0x58 , 0x16},//raw_dif_thr_normal
{0x56 , 0x18},//dummy
{0x59 , 0x18},//low
{0x5a , 0x70},//raw_rb_fac_outdoor{raw_rb_fac_nr
{0x5b , 0x00},//raw_rb_fac_dummy{raw_rb_fac_low
#endif
//sharpen
{0xfd , 0x01},
{0xe5 , 0x20},
{0xe6 , 0x10},
{0xe7 , 0x20},
{0xe8 , 0x38},//sharp_fac_pos_outdoor
{0xec , 0x38},//sharp_fac_neg_outdoor
{0xe9 , 0x30},//sharp_fac_pos_nr
{0xed , 0x30},//sharp_fac_neg_nr
{0xea , 0x30},//sharp_fac_pos_dummy
{0xef , 0x30},//sharp_fac_neg_dummy
{0xeb , 0x10},//sharp_fac_pos_low
{0xf0 , 0x20},//sharp_fac_neg_low
//rpc
{0xfd , 0x00},                
{0xe0 , 0x32},//rpc_1base_max 
{0xe1 , 0x26},//rpc_2base_max 
{0xe2 , 0x22},//rpc_3base_max 
{0xe3 , 0x1e},//rpc_4base_max 
{0xe4 , 0x1e},//rpc_5base_max 
{0xe5 , 0x1d},//rpc_6base_max 
{0xe6 , 0x1d},//rpc_7base_max 
{0xe8 , 0x1b},//rpc_8base_max 
{0xe9 , 0x1b},//rpc_9base_max 
{0xea , 0x1b},//rpc_10base_max
{0xeb , 0x1a},//rpc_11base_max
{0xf5 , 0x1a},//rpc_12base_max
{0xf6 , 0x1a},//rpc_13base_max
//ae min gain  
{0xfd , 0x01},
{0x95 , 0x1a},//rpc_min_indr 
{0x9d , 0x1a},//rpc_min_outdr
//ae setting 24MHz 20~8fps maxgain:0x70
{0xfd , 0x01},
{0x94 , 0x70},//rpc_max_indr
{0x9c , 0x70},//rpc_max_outdr
{0xfd , 0x00}, 
{0x05 , 0x00},
{0x06 , 0x00},
{0x07 , 0x00},
{0x08 , 0x00},
{0x09 , 0x01},
{0x0A , 0x76},
{0xf0 , 0x62},//ABF_exp_base_8lsb
{0xf1 , 0x00},//exp_min_base_msb
{0xfd , 0x01},
{0x90 , 0x98},//exp_max_indr[7:0]
{0x91 , 0x04},//exp_max_indr[12:8]
{0x92 , 0x62},//exp_min_indr[7:0]
{0x93 , 0x00},//exp_min_indr[12:8] 
{0x98 , 0x62},//exp_max_outdr[7:0]
{0x99 , 0x00},//exp_max_outdr[12:8]
{0x9a , 0x01},//exp_min_outdr[7:0]
{0x9b , 0x00},//exp_min_outdr[12:8]
{0xfd , 0x01},//Status
{0xc4 , 0x70},//mean_nr_dummy
{0xc5 , 0x74},//mean_dummy_nr
{0xca , 0x30},//mean_heq_low
{0xcb , 0x45},//mean_heq_dummy
{0xcc , 0x70},//rpc_heq_low
{0xcd , 0x70},//rpc_heq_dummy
{0xce , 0x98},//exp_heq_dummy_8lsm
{0xcf , 0x04},//exp_heq_dummy_5hsm
{0xd0 , 0x98},//exp_heq_low_8lsm
{0xd1 , 0x04},//exp_heq_low_5hsm
//ae target
{0xfd , 0x00}, 
{0xed , Pre_Value_P0_0xf7+0x04},
{0xf7 , Pre_Value_P0_0xf7},
{0xf8 , Pre_Value_P0_0xf8},
{0xec , Pre_Value_P0_0xf8-0x04},
{0xef , Pre_Value_P0_0xf9+0x04},
{0xf9 , Pre_Value_P0_0xf9},
{0xfa , Pre_Value_P0_0xfa},
{0xee , Pre_Value_P0_0xfa-0x04},

              
/*// gamma 1 ͹ϰ
{0xfd , 0x00},                 
{0x8b , 0x0 },                 
{0x8c , 0x6 },                 
{0x8d , 0xC },                 
{0x8e , 0x1A},                 
{0x8f , 0x3A},                 
{0x90 , 0x56},                 
{0x91 , 0x70},                 
{0x92 , 0x86},                 
{0x93 , 0x99},                 
{0x94 , 0xA9},                 
{0x95 , 0xB5},                 
{0x96 , 0xC0},                 
{0x97 , 0xCA},                 
{0x98 , 0xD4},                 
{0x99 , 0xDD},                 
{0x9a , 0xE6},                 
{0x9b , 0xEF},                 
{0xfd , 0x01},                 
{0x8d , 0xF7},                 
{0x8e , 0xFF},*/                 
/*// gamma 2 ͹԰   
{0xfd , 0x00},                 
{0x8b , 0x0 },                 
{0x8c , 0xA },                 
{0x8d , 0x13},                 
{0x8e , 0x25},                 
{0x8f , 0x43},                 
{0x90 , 0x5D},                 
{0x91 , 0x74},                 
{0x92 , 0x88},                 
{0x93 , 0x9A},                 
{0x94 , 0xA9},                 
{0x95 , 0xB5},                 
{0x96 , 0xC0},                 
{0x97 , 0xCA},                 
{0x98 , 0xD4},                 
{0x99 , 0xDD},                 
{0x9a , 0xE6},                 
{0x9b , 0xEF},                 
{0xfd , 0x01},                 
{0x8d , 0xF7},                 
{0x8e , 0xFF},*/  
//gamma  3 Ƽʹ
{0xfd , 0x00},                 
{0x8b , 0x0 },                 
{0x8c , 0xC },                 
{0x8d , 0x19},                 
{0x8e , 0x2C},                 
{0x8f , 0x49},                 
{0x90 , 0x61},                 
{0x91 , 0x77},                 
{0x92 , 0x8A},                 
{0x93 , 0x9B},                 
{0x94 , 0xA9},                 
{0x95 , 0xB5},                 
{0x96 , 0xC0},                 
{0x97 , 0xCA},                 
{0x98 , 0xD4},                 
{0x99 , 0xDD},                 
{0x9a , 0xE6},                 
{0x9b , 0xEF},                 
{0xfd , 0x01},                 
{0x8d , 0xF7},                 
{0x8e , 0xFF},   
/*// gamma 4 ͹
{0xfd , 0x00},                 
{0x8b , 0x0 },                 
{0x8c , 0x14},                 
{0x8d , 0x22},                 
{0x8e , 0x36},                 
{0x8f , 0x51},                 
{0x90 , 0x67},                 
{0x91 , 0x7A},                 
{0x92 , 0x8C},                 
{0x93 , 0x9B},                 
{0x94 , 0xA9},                 
{0x95 , 0xB5},                 
{0x96 , 0xC0},                 
{0x97 , 0xCA},                 
{0x98 , 0xD4},                 
{0x99 , 0xDD},                 
{0x9a , 0xE6},                 
{0x9b , 0xEF},                 
{0xfd , 0x01},                 
{0x8d , 0xF7},                 
{0x8e , 0xFF},*/        
/*// gamma 5 ͹
{0xfd , 0x00},                 
{0x8b , 0x0 },                 
{0x8c , 0x20},                 
{0x8d , 0x2d},                 
{0x8e , 0x40},                 
{0x8f , 0x5a},                 
{0x90 , 0x6e},                 
{0x91 , 0x80},                 
{0x92 , 0x8f},                 
{0x93 , 0x9c},                 
{0x94 , 0xa9},                 
{0x95 , 0xb5},                 
{0x96 , 0xc0},                 
{0x97 , 0xca},                 
{0x98 , 0xd4},                 
{0x99 , 0xdd},                 
{0x9a , 0xe6},                 
{0x9b , 0xef},                 
{0xfd , 0x01},                 
{0x8d , 0xf7},                 
{0x8e , 0xff},*/    

//Lens shading  
{0xfd , 0x00},
{0xa0 , 0x10},//lsc_gr_gain
{0xa1 , 0x10},//lsc_gb_gain
{0xa2 , 0x14},//lsc_r_gain
{0xa3 , 0x10},//lsc_b_gain
{0xa4 , 0x14},//lsc_gain_max
{0xa5 , 0x80},//col_center
{0xa6 , 0x80},//row_center                  
//uv en
{0xfd , 0x00}, 
{0xb0 , 0x00},//uv_dif_gain_outdoor{uv_dif_gain_nr
{0xb1 , 0x33},//uv_dif_gain_dummy{uv_dif_gain_low    
//neg
{0xfd , 0x01},
{0xd2 , 0x00}, 
//lowlight lum
{0xfd , 0x00}, 
{0xb2 , NORMAL_Y0ffset},//lum_limit
{0xb3 , 0x1f},//lum_set
{0xb4 , 0x20},//black_vt
{0xb5 , 0x45},//white_vt
//auto sat
{0xfd , 0x00},
{0xbf , 0x01},//autosa_en
{0xc0 , 0xff},//y_p_th
{0xc1 , 0xff},//y_mean_th
//saturation
{0xfd , 0x00}, 
{0xd8 , 0x80},//sat_uv_s1
{0xd9 , 0x80},//sat_uv_s2
{0xda , 0x80},//sat_uv_s3
{0xdb , 0x80},//sat_uv_s4
//heq
{0xfd , 0x00},
{0xdc , 0x00},//heq_offset
{0xdd , Pre_Value_P0_0xdd},//ku
{0xde , Pre_Value_P0_0xde},//kl
{0xdf , 0x70},//heq_mean
//CCM
{0xfd , 0x01},//        
{0xa0 , 0x80},//80(ɫӽɫ)         
{0xa1 , 0x0 },//0                                
{0xa2 , 0x0 },//0                                
{0xa3 , 0xf3},//f0                               
{0xa4 , 0x8e},//a6                               
{0xa5 , 0x0 },//ea                               
{0xa6 , 0x0 },//0                                
{0xa7 , 0xe6},//e6 
{0xa8 , 0x9a},//9a
{0xa9 , 0x0 },//0                                
{0xaa , 0x3 },//33                               
{0xab , 0xc },//c
//LSC
 {0xfd , 0x00},
 {0x33 , 0xcf},
//skin
{0xfd , 0x01},
{0x8b , 0x2f},//bit7,0 en
{0x01 , 0xf0},
{0x02 , 0x04},
{0x03 , 0x70},
{0x04 , 0x99},
{0x05 , 0x24},
//resize_en
{0xfd , 0x01},
{0x0e , 0x00},
//outformat
{0xfd , 0x00},
{0x35 , 0x40},
{0x5f , 0x01},
//En AE
{0xfd , 0x00},
{0x32 , 0x0d},    
{0x34 , 0x06},  
{0xf4 , 0x09}, 
{0xFF,0xFF},
};


/**---------------------------------------------------------------------------*
 **                         Function Definitions                              *
 **---------------------------------------------------------------------------*/
LOCAL SP0A18_WriteReg( uint8  subaddr, uint8 data )
{
	
	#ifndef	_USE_DSP_I2C_
		uint8 cmd[2];
		cmd[0]	=	subaddr;
		cmd[1]	=	data;
		
		I2C_WriteCmdArr(SP0A18_I2C_ADDR_W, cmd, 2, SCI_TRUE);
	#else
		DSENSOR_IICWrite((uint16)subaddr, (uint16)data);
	#endif

	SCI_TRACE_LOW("SENSOR: SP0A18_WriteReg reg/value(%x,%x) !!", subaddr, data);

}

LOCAL uint8 SP0A18_ReadReg( uint8  subaddr)
{
	uint8 value = 0;
	
	#ifndef	_USE_DSP_I2C_
		I2C_WriteCmdArr(SP0A18_I2C_ADDR_W, &subaddr, 1, SCI_TRUE);
		I2C_ReadCmd(SP0A18_I2C_ADDR_R, &value, SCI_TRUE);
		SCI_TRACE_LOW( "SP0A18 read ret is %X=%X",subaddr,value );
	#else
		value = (uint16)DSENSOR_IICRead((uint16)subaddr);
	#endif
	
	return value;
}

/******************************************************************************/
// Description: sensor probe function
// Author:     benny.zou
// Input:      none
// Output:     result
// Return:     0           successful
//             others      failed
// Note:       this function only to check whether sensor is work, not identify 
//              whitch it is!!
/******************************************************************************/
LOCAL uint32 SP0A18_Identify(void)
{
    uint8 reg[2]    = {0x02, 0x02};
    uint8 value[2]  = {0x0a, 0x0a};
    uint8 ret       = 0;
    uint32 i;
    uint8 err_cnt = 0;
    uint32 nLoop = 1000;

//	SP0A18_DELAY_MS(10);

    for(i = 0; i<2; )
    {
        nLoop = 1000;
        //SP0A18_WriteReg(0xfd,0x00);  
        ret = SP0A18_ReadReg(reg[i]);    
		
		SCI_TRACE_LOW( "SP0A18 read ret is %X",ret );
		
        if( ret != value[i])
        {
            err_cnt++;
            if(err_cnt>3)
            {
				SCI_TRACE_LOW( "SP0A18 Fail" );
				return 0xFF;
            }
            else
            {
                //Masked by frank.yang,SCI_Sleep() will cause a  Assert when called in boot precedure
                //SCI_Sleep(10);
                while(nLoop--);
                continue;
            }
        }
        err_cnt = 0;
        i++;
    }

     SCI_TRACE_LOW( "SP0A18 succ" );

return (uint32)SCI_SUCCESS;

}



LOCAL uint32 hmirror_enable(uint32 enable);
LOCAL uint32 vmirror_enable(uint32 enable);

__align(4) const SENSOR_REG_T SP0A18_brightness_tab[][3]=
{
	{
		{0xfd,0x00},
		{0xdc,0xd0},//level -3
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0xe0},//level -2
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0xf0},//level -1
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x00},//level 0
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x10},//level +1
		{0xff,0xff},
	},

	{
		{0xfd,0x00},
		{0xdc,0x20},//level +2
		{0xff,0xff},
	},
    {		
		{0xfd,0x00},
		{0xdc,0x30},//level +3
		{0xff,0xff},
	},

};
LOCAL uint32 set_brightness(uint32 level)
{
    
	uint16 i;
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_brightness_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        //Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
		SP0A18_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
    return 0;
}

__align(4) const SENSOR_REG_T SP0A18_contrast_tab[][4]=
{
	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd-0x30},	//level -3
		{0xde,  Pre_Value_P0_0xde-0x30},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd-0x20},	//level -2
		{0xde,  Pre_Value_P0_0xde-0x20},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd, Pre_Value_P0_0xdd-0x10},	//level -1
		{0xde, Pre_Value_P0_0xde-0x10},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd, Pre_Value_P0_0xdd},	//level 0
		{0xde, Pre_Value_P0_0xde},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd+0x10},	//level +1
		{0xde,  Pre_Value_P0_0xde+0x10},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd+0x20},	//level +2
		{0xde,  Pre_Value_P0_0xde+0x20},
		{0xff, 0xff}
	},

	{
		{0xfd, 0x00},
		{0xdd,  Pre_Value_P0_0xdd+0x30},	//level +3
		{0xde,  Pre_Value_P0_0xde+0x30},
		{0xff, 0xff}
	}, 
};

LOCAL uint32 set_contrast(uint32 level)
{
    uint16 i;    
    
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_contrast_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        //Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
		SP0A18_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
    return 0;
}


LOCAL uint32 Sensor_Write_Regs( SENSOR_REG_T* sensor_reg_ptr )
{

    uint32 i;
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) && (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
    return 0;
}


__align(4) const SENSOR_REG_T SP0A18_mode_tab[][4]=
{
#if 1
{    
{0xfd,0x00},
{0xfd,0x00},
{0xfd,0x00}, 
{0xFF,0xFF},
}
#else
//capture preview daylight
{     
{0xfd,0x00},
{0xb2,NORMAL_Y0ffset}, 
{0xb3,0x1f}, //lum_set
{0xFF,0xFF},	//־
},
//capture preview night
{    
{0xfd,0x00},
{0xb2,LOWLIGHT_Y0ffset}, 
{0xb3,0x1f}, //lum_set
{0xFF,0xFF},	//־ 
}
#endif
};  
    
LOCAL uint32 set_preview_mode(uint32 preview_mode)
{
	switch (preview_mode)
	{
		case DCAMERA_ENVIRONMENT_NORMAL: 
		case DCAMERA_ENVIRONMENT_SUNNY:
		{
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0xb2,NORMAL_Y0ffset);
			SP0A18_WriteReg(0xb3,0x1f);
			if(Antiflicker== DCAMERA_FLICKER_50HZ)
			{
			SCI_TRACE_LOW("normal mode 50hz\r\n");
			#if CAPPRE_20_10FPS
			//capture preview daylight 24M 50hz 20-10FPS maxgain:0x78
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_10FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_10FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x76);
			SP0A18_WriteReg(0xf0,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0xd4);
			SP0A18_WriteReg(0x91,0x3 );
			SP0A18_WriteReg(0x92,0x62);
			SP0A18_WriteReg(0x98,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_10FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_10FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0xd4);
			SP0A18_WriteReg(0xcf,0x3 );
			SP0A18_WriteReg(0xd0,0xd4);
			SP0A18_WriteReg(0xd1,0x3 );
			#endif				
			#if CAPPRE_20_09FPS
			//capture preview daylight 24M 50hz 20-9FPS maxgain:0x74
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_09FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_09FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x76);
			SP0A18_WriteReg(0xf0,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x36);
			SP0A18_WriteReg(0x91,0x4 );
			SP0A18_WriteReg(0x92,0x62);
			SP0A18_WriteReg(0x98,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_09FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_09FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0x36);
			SP0A18_WriteReg(0xcf,0x4 );
			SP0A18_WriteReg(0xd0,0x36);
			SP0A18_WriteReg(0xd1,0x4 );
			#endif		
			#if CAPPRE_20_08FPS
			//capture preview daylight 24M 50hz 20-8FPS maxgain:0x70   
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_08FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_08FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x76);
			SP0A18_WriteReg(0xf0,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x98);
			SP0A18_WriteReg(0x91,0x4 );
			SP0A18_WriteReg(0x92,0x62);
			SP0A18_WriteReg(0x98,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_08FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_08FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0x98);
			SP0A18_WriteReg(0xcf,0x4 );
			SP0A18_WriteReg(0xd0,0x98);
			SP0A18_WriteReg(0xd1,0x4 );
			#endif
			#if CAPPRE_20_07FPS
			//capture preview daylight 24M 50hz 20-7FPS maxgain:0x6c
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_07FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_07FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x76);
			SP0A18_WriteReg(0xf0,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x5c);
			SP0A18_WriteReg(0x91,0x5 );
			SP0A18_WriteReg(0x92,0x62);
			SP0A18_WriteReg(0x98,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_07FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_07FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0x5c);
			SP0A18_WriteReg(0xcf,0x5 );
			SP0A18_WriteReg(0xd0,0x5c);
			SP0A18_WriteReg(0xd1,0x5 );
			#endif
			#if CAPPRE_20_06FPS
			//capture preview daylight 24M 50hz 20-6FPS maxgain:0x68
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_06FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_06FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x76);
			SP0A18_WriteReg(0xf0,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x20);
			SP0A18_WriteReg(0x91,0x6 );
			SP0A18_WriteReg(0x92,0x62);
			SP0A18_WriteReg(0x98,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_06FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_06FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0x20);
			SP0A18_WriteReg(0xcf,0x6 );
			SP0A18_WriteReg(0xd0,0x20);
			SP0A18_WriteReg(0xd1,0x6 );
			#endif	
			}
			else	
			{
			SCI_TRACE_LOW("normal mode 60hz\r\n");
			#if CAPPRE_20_10FPS
			//capture preview daylight 24M 60hz 20-10FPS maxgain:0x78
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_10FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_10FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x72);
			SP0A18_WriteReg(0xf0,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0xd8);
			SP0A18_WriteReg(0x91,0x3 );
			SP0A18_WriteReg(0x92,0x52);
			SP0A18_WriteReg(0x98,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_10FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_10FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0xd8);
			SP0A18_WriteReg(0xcf,0x3 );
			SP0A18_WriteReg(0xd0,0xd8);
			SP0A18_WriteReg(0xd1,0x3 );
			#endif				
			#if CAPPRE_20_09FPS
			//capture preview daylight 24M 60hz 20-9FPS maxgain:0x74
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_09FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_09FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x72);
			SP0A18_WriteReg(0xf0,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x2a);
			SP0A18_WriteReg(0x91,0x4 );
			SP0A18_WriteReg(0x92,0x52);
			SP0A18_WriteReg(0x98,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_09FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_09FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0x2a);
			SP0A18_WriteReg(0xcf,0x4 );
			SP0A18_WriteReg(0xd0,0x2a);
			SP0A18_WriteReg(0xd1,0x4 );
			#endif		
			#if CAPPRE_20_08FPS
			//capture preview daylight 24M 60hz 20-8FPS maxgain:0x70   
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_08FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_08FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x72);
			SP0A18_WriteReg(0xf0,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0xce);
			SP0A18_WriteReg(0x91,0x4 );
			SP0A18_WriteReg(0x92,0x52);
			SP0A18_WriteReg(0x98,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_08FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_08FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0xce);
			SP0A18_WriteReg(0xcf,0x4 );
			SP0A18_WriteReg(0xd0,0xce);
			SP0A18_WriteReg(0xd1,0x4 );
			#endif
			#if CAPPRE_20_07FPS
			//capture preview daylight 24M 60hz 20-7FPS maxgain:0x6c
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_07FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_07FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x72);
			SP0A18_WriteReg(0xf0,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x72);
			SP0A18_WriteReg(0x91,0x5 );
			SP0A18_WriteReg(0x92,0x52);
			SP0A18_WriteReg(0x98,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_07FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_07FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0x72);
			SP0A18_WriteReg(0xcf,0x5 );
			SP0A18_WriteReg(0xd0,0x72);
			SP0A18_WriteReg(0xd1,0x5 );
			#endif
			#if CAPPRE_20_06FPS
			//capture preview daylight 24M 60hz 20-6FPS maxgain:0x68
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_06FPS_MAXGAIN);
			SP0A18_WriteReg(0x9c,CAPPRE_20_06FPS_MAXGAIN);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x72);
			SP0A18_WriteReg(0xf0,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x68);
			SP0A18_WriteReg(0x91,0x6 );
			SP0A18_WriteReg(0x92,0x52);
			SP0A18_WriteReg(0x98,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_06FPS_MAXGAIN);
			SP0A18_WriteReg(0xcd,CAPPRE_20_06FPS_MAXGAIN);
			SP0A18_WriteReg(0xce,0x68);
			SP0A18_WriteReg(0xcf,0x6 );
			SP0A18_WriteReg(0xd0,0x68);
			SP0A18_WriteReg(0xd1,0x6 );
			#endif				
			}	
			break;
		}
		case DCAMERA_ENVIRONMENT_NIGHT:
		{
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0xb2,LOWLIGHT_Y0ffset);
			SP0A18_WriteReg(0xb3,0x1f);
			if(Antiflicker== DCAMERA_FLICKER_50HZ)
			{
			SCI_TRACE_LOW("night mode 50hz\r\n");
			#if CAPPRE_20_06FPS_NIGHT
			//capture preview night 24M 50hz 20-6FPS maxgain:0x78	 
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_06FPS_MAXGAIN_NIGHT);
			SP0A18_WriteReg(0x9c,CAPPRE_20_06FPS_MAXGAIN_NIGHT);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x76);
			SP0A18_WriteReg(0xf0,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x20);
			SP0A18_WriteReg(0x91,0x6 );
			SP0A18_WriteReg(0x92,0x62);
			SP0A18_WriteReg(0x98,0x62);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_06FPS_MAXGAIN_NIGHT);
			SP0A18_WriteReg(0xcd,CAPPRE_20_06FPS_MAXGAIN_NIGHT);
			SP0A18_WriteReg(0xce,0x20);
			SP0A18_WriteReg(0xcf,0x6 );
			SP0A18_WriteReg(0xd0,0x20);
			SP0A18_WriteReg(0xd1,0x6 );
                     #endif
			}
			else
			{
			SCI_TRACE_LOW("night mode 60hz\r\n");
			#if CAPPRE_20_06FPS_NIGHT
			//capture preview night 24M 60hz 20-6FPS maxgain:0x78
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x94,CAPPRE_20_06FPS_MAXGAIN_NIGHT);
			SP0A18_WriteReg(0x9c,CAPPRE_20_06FPS_MAXGAIN_NIGHT);
			SP0A18_WriteReg(0xfd,0x0 );
			SP0A18_WriteReg(0x09,0x1 );
			SP0A18_WriteReg(0x0a,0x72);
			SP0A18_WriteReg(0xf0,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0x90,0x68);
			SP0A18_WriteReg(0x91,0x6 );
			SP0A18_WriteReg(0x92,0x52);
			SP0A18_WriteReg(0x98,0x52);
			SP0A18_WriteReg(0xfd,0x1 );
			SP0A18_WriteReg(0xcc,CAPPRE_20_06FPS_MAXGAIN_NIGHT);
			SP0A18_WriteReg(0xcd,CAPPRE_20_06FPS_MAXGAIN_NIGHT);
			SP0A18_WriteReg(0xce,0x68);
			SP0A18_WriteReg(0xcf,0x6 );
			SP0A18_WriteReg(0xd0,0x68);
			SP0A18_WriteReg(0xd1,0x6 );
                     #endif
			}
			break;
		}
		
		default:
		{
			break;
		}

	SCI_Sleep(100);
return 0;
	} 
}


__align(4) const SENSOR_REG_T SP0A18_image_effect_tab[][5]=	
{
    // effect normal
    {
		{0xfd, 0x00},
		{0x62, 0x00},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },
    //effect BLACKWHITE//GRAYSCALE
    {
		{0xfd, 0x00},
		{0x62, 0x20},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },
    // effect RED pink
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0xb0},
		{0x64, 0x90},
		{0xff, 0xff}
    },
    // effect GREEN//SEPIAGREEN
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0x20},
		{0x64, 0x20},
		{0xff, 0xff}
    },
    // effect  BLUE//SEPIABLUE
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0x20},
		{0x64, 0xf0},
		{0xff, 0xff}
    },
    // effect  YELLOW//SEPIAYELLOW
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0xb0},
		{0x64, 0x00},
		{0xff, 0xff}

    },  
    // effect NEGATIVE//COLORINV
    {	     
		{0xfd, 0x00},
		{0x62, 0x04},
		{0x63, 0x80},
		{0x64, 0x80},
		{0xff, 0xff}
    },    
    //effect ANTIQUE//SEPIA
    {
		{0xfd, 0x00},
		{0x62, 0x10},
		{0x63, 0xc0},
		{0x64, 0x20},
		{0xff, 0xff}
    },
};
LOCAL uint32 set_image_effect(uint32 effect_type)
{
    uint16 i;
    
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_image_effect_tab[effect_type];

    SCI_ASSERT(PNULL != sensor_reg_ptr);

    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        //Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
		SP0A18_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
    return 0;
}

LOCAL uint32 SP0A18_ae_awb_enable(uint32 ae_enable, uint32 awb_enable)
{
	uint16 ae_value = 0 , awb_value=0;

	SP0A18_WriteReg(0xfd,0x00);
	ae_value = SP0A18_ReadReg(0x32);
	awb_value = SP0A18_ReadReg(0x32);

	if(0x01==ae_enable)
	{
		ae_value |= 0x05;       
	}
	else if(0x00==ae_enable)
	{
		ae_value &= 0xf8;
	}

	if(0x01==awb_enable)
	{
		awb_value |= 0x08;
	}
	else if(0x00==awb_enable)
	{
		awb_value &= 0xf7;
	}	
	
	SP0A18_WriteReg(0x32, ae_value);
	SP0A18_WriteReg(0x32, awb_value);
	return 0;
}

LOCAL void Delayms (uint32 ms)	
{
	uint32 t1, t2;
	
	t1 = t2 = SCI_GetTickCount ();
	
	do{
		t2 = SCI_GetTickCount ();
	}while (t2 < (t1+ms));
}

LOCAL uint32 SP0A18_before_snapshot(uint32 para)
{
	#if 0                   // for FAE when have to reconfig the HB/VB.
	uint8 reg_val = 0;
	uint16 exposal_time=0x00;


	SP0A18_ae_awb_enable(0x00,0x00);   // close aec awb

	reg_val = SP0A18_ReadReg(0x04);
	exposal_time=reg_val&0x00ff;
	reg_val = SP0A18_ReadReg(0x03);
	exposal_time|=((reg_val&0x00ff)<<0x08);
	
	//exposal_time=exposal_time*2/3;

	if(exposal_time<2)
	{
		exposal_time=2;
	}

	reg_val=exposal_time&0x00ff;
	SP0A18_WriteReg(0x04, reg_val);
	reg_val=(exposal_time&0xff00)>>0x08;
	SP0A18_WriteReg(0x03, reg_val);

	
	SP0A18_WriteReg(0x01,0xfa);
	SP0A18_WriteReg(0x02,0x70);  
	SP0A18_WriteReg(0x0f,0x01);
	SP0A18_WriteReg(0xe2,0x00);
	SP0A18_WriteReg(0xe3,0x64);
	
	Delayms(400);	
	#endif

	return 0;
}

LOCAL uint32 SP0A18_after_snapshot(uint32 para)
{

	#if 0                          // for FAE when have to reconfig the HB/VB.
	SP0A18_WriteReg(0x01,0xfa);  
	SP0A18_WriteReg(0x02,0x70);  
	SP0A18_WriteReg(0x0f,0x01);
	SP0A18_WriteReg(0xe2,0x00);
	SP0A18_WriteReg(0xe3,0x64);  


	SP0A18_ae_awb_enable(0x01,0x01);   // Open aec awb
	
	Delayms(400);
	#endif

	return 0;
}

//@ Chenfeng for adding AWB & AE functions
__align(4) const SENSOR_REG_T SP0A18_awb_tab[][7]=
{
	   	//AUTO  // AUTO 3000K~7000K 
	   	{
			{0xfd,0x01},                      
			{0x28,Pre_Value_P1_0x28},		                  
			{0x29,Pre_Value_P1_0x29},                      
			{0xfd,0x00}, 
			{0x32,0x0d},   
			{0xfd,0x00},  
		{0xff,0xff} 
		},    
		//INCANDESCENCE:  //2800K~3000K  
		{
			{0xfd,0x00}, 
			{0x32,0x05},                 
			{0xfd,0x01},                 
			{0x28,0x7b},		             
			{0x29,0xd3},		             
			{0xfd,0x00}, 
		{0xff,0xff} 
		},
		//U30 //3000K
		{
			{0xfd,0x00},
			{0x32,0x05},       
			{0xfd,0x01},       
			{0x28,0x82},	     
			{0x29,0xe2},	     
			{0xfd,0x00},     
		{0xff,0xff} 
		},  
		//CWF  //4150K
		{
			{0xfd,0x00},
			{0x32,0x05},       
			{0xfd,0x01},       
			{0x28,0xa3},	     
			{0x29,0xb3},	     
			{0xfd,0x00},     
		{0xff,0xff} 
		},    
		//FLUORESCENT://4200K~5000K   
		{
			{0xfd,0x00},  
			{0x32,0x05},                  
			{0xfd,0x01},                  
			{0x28,0xb4},		              
			{0x29,0xc4},		              
			{0xfd,0x00},
		    {0xff,0xff} 
		},
		//SUN: //DAYLIGHT//6500K   
		{
			{0xfd,0x00}, 
			{0x32,0x05},           
			{0xfd,0x01},           
			{0x28,0xc1},	         
			{0x29,0x88},	         
			{0xfd,0x00},  
		    {0xff,0xff} 
		},
                //CLOUD: //7000K 
		{
			{0xfd,0x00}, 
			{0x32,0x05},          
			{0xfd,0x01},          
			{0x28,0xe2},	        
			{0x29,0x82},	        
			{0xfd,0x00},
		    {0xff,0xff} 
		},
};

LOCAL uint32 set_SP0A18_awb(uint32 mode)
{
	uint8 awb_en_value;
	uint16 i;
	
	SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_awb_tab[mode];

	SP0A18_WriteReg(0xfd,0x00);
	awb_en_value = SP0A18_ReadReg(0x32);

	SCI_ASSERT(mode < 7);
	SCI_ASSERT(PNULL != sensor_reg_ptr);
	
	for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value); i++)
	{
	       if(0x32 == sensor_reg_ptr[i].reg_addr)
       	{
			if(mode == 0)
		   		SP0A18_WriteReg(0x32, awb_en_value |0x08 );
			else
               	SP0A18_WriteReg(0x32, awb_en_value &0xf7 );
       	}
      else
       {
           SP0A18_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
	   }
	}
	Delayms(100); 
	return 0;
}

LOCAL uint32 set_SP0A18_ae_enable(uint32 enable)
{
   #define AE_ENABLE  (0x32)
    unsigned char ae_value;
		SP0A18_WriteReg(0xfd,0x00);
        ae_value=SP0A18_ReadReg(AE_ENABLE);

        if(0x00==enable)
        {
            ae_value &= 0xf8;
            SP0A18_WriteReg(AE_ENABLE,ae_value);
        }
        else if(0x01==enable)
        {
            ae_value|=0x05;
            SP0A18_WriteReg(AE_ENABLE,ae_value);
        }

	return 0;
}

/******************************************************************************/
// Description: anti 50/60 hz banding flicker
// Global resource dependence: 
// Author:
// Note:
//		
/******************************************************************************/
LOCAL uint32 set_SP0A18_anti_flicker(uint32 mode)
{ 
     switch(mode)
    {
        case DCAMERA_FLICKER_50HZ:			
		Antiflicker = DCAMERA_FLICKER_50HZ;
		SCI_TRACE_LOW( " set_SP0A18_anti_flicker  50hz \r\n" );
		break;
            
        case DCAMERA_FLICKER_60HZ:
		Antiflicker = DCAMERA_FLICKER_60HZ;
		SCI_TRACE_LOW( " set_SP0A18_anti_flicker  60hz \r\n" );
		break;
            
        default:
            break;
    }
    
    Delayms(100); 
    return 0;
}
/******************************************************************************/
// Description: set video mode
// Global resource dependence: 
// Author:
// Note:
//		 
/******************************************************************************/
__align(4) const SENSOR_REG_T SP0A18_video_mode_nand_tab[][9]=
{
#if 1
    {		
		{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xfd, 0x00},{0xff,0xff},
	},

#else
    {
        {0x01,0x6a},{0x02,0x32},{0x0f,0x20},{0xe2,0x00},{0xe3,0x96},{0xe8,0x04},{0xe9,0x1a}, {0xec,0x20},{0xff, 0xff} 
    },    
    //vodeo mode
    {
        {0x01,0x6a},{0x02,0x70},{0x0f,0x00},{0xe2,0x00},{0xe3,0x96},{0xe8,0x02},{0xe9,0x58}, {0xec,0x20},{0xff, 0xff}      
    }
#endif
};
/******************************************************************************/
// Description: set video mode
// Global resource dependence: 
// Author:
// Note:
//		 
/******************************************************************************/
__align(4) const SENSOR_REG_T SP0A18_video_mode_nor_tab_50hz[][20]=
{
//Video preview
{
      #if CAPPRE_20_10FPS
			//capture preview daylight 24M 50hz 20-10FPS maxgain:0x78
			{0xfd,0x1 },
			{0x94,CAPPRE_20_10FPS_MAXGAIN},
			{0x9c,CAPPRE_20_10FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x76},
			{0xf0,0x62},
			{0xfd,0x1 },
			{0x90,0xd4},
			{0x91,0x3 },
			{0x92,0x62},
			{0x98,0x62},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_10FPS_MAXGAIN},
			{0xcd,CAPPRE_20_10FPS_MAXGAIN},
			{0xce,0xd4},
			{0xcf,0x3 },
			{0xd0,0xd4},
			{0xd1,0x3 },
			#endif				
			#if CAPPRE_20_09FPS
			//capture preview daylight 24M 50hz 20-9FPS maxgain:0x74
			{0xfd,0x1 },
			{0x94,CAPPRE_20_09FPS_MAXGAIN},
			{0x9c,CAPPRE_20_09FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x76},
			{0xf0,0x62},
			{0xfd,0x1 },
			{0x90,0x36},
			{0x91,0x4 },
			{0x92,0x62},
			{0x98,0x62},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_09FPS_MAXGAIN},
			{0xcd,CAPPRE_20_09FPS_MAXGAIN},
			{0xce,0x36},
			{0xcf,0x4 },
			{0xd0,0x36},
			{0xd1,0x4 },
			#endif		
			#if CAPPRE_20_08FPS
			//capture preview daylight 24M 50hz 20-8FPS maxgain:0x70   
			{0xfd,0x1 },
			{0x94,CAPPRE_20_08FPS_MAXGAIN},
			{0x9c,CAPPRE_20_08FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x76},
			{0xf0,0x62},
			{0xfd,0x1 },
			{0x90,0x98},
			{0x91,0x4 },
			{0x92,0x62},
			{0x98,0x62},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_08FPS_MAXGAIN},
			{0xcd,CAPPRE_20_08FPS_MAXGAIN},
			{0xce,0x98},
			{0xcf,0x4 },
			{0xd0,0x98},
			{0xd1,0x4 },
			#endif
			#if CAPPRE_20_07FPS
			//capture preview daylight 24M 50hz 20-7FPS maxgain:0x6c
			{0xfd,0x1 },
			{0x94,CAPPRE_20_07FPS_MAXGAIN},
			{0x9c,CAPPRE_20_07FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x76},
			{0xf0,0x62},
			{0xfd,0x1 },
			{0x90,0x5c},
			{0x91,0x5 },
			{0x92,0x62},
			{0x98,0x62},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_07FPS_MAXGAIN},
			{0xcd,CAPPRE_20_07FPS_MAXGAIN},
			{0xce,0x5c},
			{0xcf,0x5 },
			{0xd0,0x5c},
			{0xd1,0x5 },
			#endif
			#if CAPPRE_20_06FPS
			//capture preview daylight 24M 50hz 20-6FPS maxgain:0x68
			{0xfd,0x1 },
			{0x94,CAPPRE_20_06FPS_MAXGAIN},
			{0x9c,CAPPRE_20_06FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x76},
			{0xf0,0x62},
			{0xfd,0x1 },
			{0x90,0x20},
			{0x91,0x6 },
			{0x92,0x62},
			{0x98,0x62},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_06FPS_MAXGAIN},
			{0xcd,CAPPRE_20_06FPS_MAXGAIN},
			{0xce,0x20},
			{0xcf,0x6 },
			{0xd0,0x20},
			{0xd1,0x6 },
			#endif	    
{0xFF,0xFF},	//־	
},    
//Video record
{
      #if VDOREC_20_20FPS
			//Video record daylight 24M 50hz 20-20FPS maxgain:0x8c
			{0xfd,0x1 },
			{0x94,VDOREC_20_20FPS_MAXGAIN},
			{0x9c,VDOREC_20_20FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x76},
			{0xf0,0x62},
			{0xfd,0x1 },
			{0x90,0xea},
			{0x91,0x1 },
			{0x92,0x62},
			{0x98,0x62},
			{0xfd,0x1 },
			{0xcc,VDOREC_20_20FPS_MAXGAIN},
			{0xcd,VDOREC_20_20FPS_MAXGAIN},
			{0xce,0xea},
			{0xcf,0x1 },
			{0xd0,0xea},
			{0xd1,0x1 },
			#endif				
			#if VDOREC_18_18FPS
			//Video record daylight 24M 50hz 18-18FPS maxgain:0x88
			{0xfd,0x1 },
			{0x94,VDOREC_18_18FPS_MAXGAIN},
			{0x9c,VDOREC_18_18FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0xf2},
			{0xf0,0x59},
			{0xfd,0x1 },
			{0x90,0xbd},
			{0x91,0x1 },
			{0x92,0x59},
			{0x98,0x59},
			{0xfd,0x1 },
			{0xcc,VDOREC_18_18FPS_MAXGAIN},
			{0xcd,VDOREC_18_18FPS_MAXGAIN},
			{0xce,0xbd},
			{0xcf,0x1 },
			{0xd0,0xbd},
			{0xd1,0x1 },
			#endif		
			#if VDOREC_16_16FPS
			//Video record daylight 24M 50hz 16-16FPS maxgain:0x84   
			{0xfd,0x1 },
			{0x94,VDOREC_16_16FPS_MAXGAIN},
			{0x9c,VDOREC_16_16FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x2 },
			{0x0a,0x9d},
			{0xf0,0x4f},
			{0xfd,0x1 },
			{0x90,0xda},
			{0x91,0x1 },
			{0x92,0x4f},
			{0x98,0x4f},
			{0xfd,0x1 },
			{0xcc,VDOREC_16_16FPS_MAXGAIN},
			{0xcd,VDOREC_16_16FPS_MAXGAIN},
			{0xce,0xda},
			{0xcf,0x1 },
			{0xd0,0xda},
			{0xd1,0x1 },
			#endif
			#if VDOREC_14_14FPS
			//Video record daylight 24M 50hz 14-14FPS maxgain:0x80
			{0xfd,0x1 },
			{0x94,VDOREC_14_14FPS_MAXGAIN},
			{0x9c,VDOREC_14_14FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x3 },
			{0x0a,0x79},
			{0xf0,0x45},
			{0xfd,0x1 },
			{0x90,0xe3},
			{0x91,0x1 },
			{0x92,0x45},
			{0x98,0x45},
			{0xfd,0x1 },
			{0xcc,VDOREC_14_14FPS_MAXGAIN},
			{0xcd,VDOREC_14_14FPS_MAXGAIN},
			{0xce,0xe3},
			{0xcf,0x1 },
			{0xd0,0xe3},
			{0xd1,0x1 },
			#endif
			#if VDOREC_12_12FPS
			//Video record daylight 24M 50hz 12-12FPS maxgain:0x7c
			{0xfd,0x1 },
			{0x94,VDOREC_12_12FPS_MAXGAIN},
			{0x9c,VDOREC_12_12FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x4 },
			{0x0a,0xa0},
			{0xf0,0x3b},
			{0xfd,0x1 },
			{0x90,0xd8},
			{0x91,0x1 },
			{0x92,0x3b},
			{0x98,0x3b},
			{0xfd,0x1 },
			{0xcc,VDOREC_12_12FPS_MAXGAIN},
			{0xcd,VDOREC_12_12FPS_MAXGAIN},
			{0xce,0xd8},
			{0xcf,0x1 },
			{0xd0,0xd8},
			{0xd1,0x1 },
			#endif	             
{0xFF,0xFF},	//־		
},
// UPCC  mode
{
      #if VDOREC_20_20FPS
			//Video record daylight 24M 50hz 20-20FPS maxgain:0x8c
			{0xfd,0x1 },
			{0x94,VDOREC_20_20FPS_MAXGAIN},
			{0x9c,VDOREC_20_20FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x76},
			{0xf0,0x62},
			{0xfd,0x1 },
			{0x90,0xea},
			{0x91,0x1 },
			{0x92,0x62},
			{0x98,0x62},
			{0xfd,0x1 },
			{0xcc,VDOREC_20_20FPS_MAXGAIN},
			{0xcd,VDOREC_20_20FPS_MAXGAIN},
			{0xce,0xea},
			{0xcf,0x1 },
			{0xd0,0xea},
			{0xd1,0x1 },
			#endif				
			#if VDOREC_18_18FPS
			//Video record daylight 24M 50hz 18-18FPS maxgain:0x88
			{0xfd,0x1 },
			{0x94,VDOREC_18_18FPS_MAXGAIN},
			{0x9c,VDOREC_18_18FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0xf2},
			{0xf0,0x59},
			{0xfd,0x1 },
			{0x90,0xbd},
			{0x91,0x1 },
			{0x92,0x59},
			{0x98,0x59},
			{0xfd,0x1 },
			{0xcc,VDOREC_18_18FPS_MAXGAIN},
			{0xcd,VDOREC_18_18FPS_MAXGAIN},
			{0xce,0xbd},
			{0xcf,0x1 },
			{0xd0,0xbd},
			{0xd1,0x1 },
			#endif		
			#if VDOREC_16_16FPS
			//Video record daylight 24M 50hz 16-16FPS maxgain:0x84   
			{0xfd,0x1 },
			{0x94,VDOREC_16_16FPS_MAXGAIN},
			{0x9c,VDOREC_16_16FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x2 },
			{0x0a,0x9d},
			{0xf0,0x4f},
			{0xfd,0x1 },
			{0x90,0xda},
			{0x91,0x1 },
			{0x92,0x4f},
			{0x98,0x4f},
			{0xfd,0x1 },
			{0xcc,VDOREC_16_16FPS_MAXGAIN},
			{0xcd,VDOREC_16_16FPS_MAXGAIN},
			{0xce,0xda},
			{0xcf,0x1 },
			{0xd0,0xda},
			{0xd1,0x1 },
			#endif
			#if VDOREC_14_14FPS
			//Video record daylight 24M 50hz 14-14FPS maxgain:0x80
			{0xfd,0x1 },
			{0x94,VDOREC_14_14FPS_MAXGAIN},
			{0x9c,VDOREC_14_14FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x3 },
			{0x0a,0x79},
			{0xf0,0x45},
			{0xfd,0x1 },
			{0x90,0xe3},
			{0x91,0x1 },
			{0x92,0x45},
			{0x98,0x45},
			{0xfd,0x1 },
			{0xcc,VDOREC_14_14FPS_MAXGAIN},
			{0xcd,VDOREC_14_14FPS_MAXGAIN},
			{0xce,0xe3},
			{0xcf,0x1 },
			{0xd0,0xe3},
			{0xd1,0x1 },
			#endif
			#if VDOREC_12_12FPS
			//Video record daylight 24M 50hz 12-12FPS maxgain:0x7c
			{0xfd,0x1 },
			{0x94,VDOREC_12_12FPS_MAXGAIN},
			{0x9c,VDOREC_12_12FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x4 },
			{0x0a,0xa0},
			{0xf0,0x3b},
			{0xfd,0x1 },
			{0x90,0xd8},
			{0x91,0x1 },
			{0x92,0x3b},
			{0x98,0x3b},
			{0xfd,0x1 },
			{0xcc,VDOREC_12_12FPS_MAXGAIN},
			{0xcd,VDOREC_12_12FPS_MAXGAIN},
			{0xce,0xd8},
			{0xcf,0x1 },
			{0xd0,0xd8},
			{0xd1,0x1 },
			#endif	             
{0xFF,0xFF},	//־	
}  
};    


__align(4) const SENSOR_REG_T SP0A18_video_mode_nor_tab_60hz[][20]=
{
//Video preview
{
      #if CAPPRE_20_10FPS
			//capture preview daylight 24M 60Hz 20-10FPS maxgain:0x78
			{0xfd,0x1 },
			{0x94,CAPPRE_20_10FPS_MAXGAIN},
			{0x9c,CAPPRE_20_10FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x72},
			{0xf0,0x52},
			{0xfd,0x1 },
			{0x90,0xd8},
			{0x91,0x3 },
			{0x92,0x52},
			{0x98,0x52},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_10FPS_MAXGAIN},
			{0xcd,CAPPRE_20_10FPS_MAXGAIN},
			{0xce,0xd8},
			{0xcf,0x3 },
			{0xd0,0xd8},
			{0xd1,0x3 },
			#endif				
			#if CAPPRE_20_09FPS
			//capture preview daylight 24M 60Hz 20-9FPS maxgain:0x74
			{0xfd,0x1 },
			{0x94,CAPPRE_20_09FPS_MAXGAIN},
			{0x9c,CAPPRE_20_09FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x72},
			{0xf0,0x52},
			{0xfd,0x1 },
			{0x90,0x2a},
			{0x91,0x4 },
			{0x92,0x52},
			{0x98,0x52},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_09FPS_MAXGAIN},
			{0xcd,CAPPRE_20_09FPS_MAXGAIN},
			{0xce,0x2a},
			{0xcf,0x4 },
			{0xd0,0x2a},
			{0xd1,0x4 },
			#endif		
			#if CAPPRE_20_08FPS
			//capture preview daylight 24M 60Hz 20-8FPS maxgain:0x70   
			{0xfd,0x1 },
			{0x94,CAPPRE_20_08FPS_MAXGAIN},
			{0x9c,CAPPRE_20_08FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x72},
			{0xf0,0x52},
			{0xfd,0x1 },
			{0x90,0xce},
			{0x91,0x4 },
			{0x92,0x52},
			{0x98,0x52},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_08FPS_MAXGAIN},
			{0xcd,CAPPRE_20_08FPS_MAXGAIN},
			{0xce,0xce},
			{0xcf,0x4 },
			{0xd0,0xce},
			{0xd1,0x4 },
			#endif
			#if CAPPRE_20_07FPS
			//capture preview daylight 24M 60Hz 20-7FPS maxgain:0x6c
			{0xfd,0x1 },
			{0x94,CAPPRE_20_07FPS_MAXGAIN},
			{0x9c,CAPPRE_20_07FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x72},
			{0xf0,0x52},
			{0xfd,0x1 },
			{0x90,0x72},
			{0x91,0x5 },
			{0x92,0x52},
			{0x98,0x52},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_07FPS_MAXGAIN},
			{0xcd,CAPPRE_20_07FPS_MAXGAIN},
			{0xce,0x72},
			{0xcf,0x5 },
			{0xd0,0x72},
			{0xd1,0x5 },
			#endif
			#if CAPPRE_20_06FPS
			//capture preview daylight 24M 60Hz 20-6FPS maxgain:0x68
			{0xfd,0x1 },
			{0x94,CAPPRE_20_06FPS_MAXGAIN},
			{0x9c,CAPPRE_20_06FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x72},
			{0xf0,0x52},
			{0xfd,0x1 },
			{0x90,0x68},
			{0x91,0x6 },
			{0x92,0x52},
			{0x98,0x52},
			{0xfd,0x1 },
			{0xcc,CAPPRE_20_06FPS_MAXGAIN},
			{0xcd,CAPPRE_20_06FPS_MAXGAIN},
			{0xce,0x68},
			{0xcf,0x6 },
			{0xd0,0x68},
			{0xd1,0x6 },
			#endif	   
{0xFF,0xFF},	//־
},    
//Video record
{
      #if VDOREC_20_20FPS
			//Video record daylight 24M 60Hz 20-20FPS maxgain:0x8c
			{0xfd,0x1 },
			{0x94,VDOREC_20_20FPS_MAXGAIN},
			{0x9c,VDOREC_20_20FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x72},
			{0xf0,0x52},
			{0xfd,0x1 },
			{0x90,0xec},
			{0x91,0x1 },
			{0x92,0x52},
			{0x98,0x52},
			{0xfd,0x1 },
			{0xcc,VDOREC_20_20FPS_MAXGAIN},
			{0xcd,VDOREC_20_20FPS_MAXGAIN},
			{0xce,0xec},
			{0xcf,0x1 },
			{0xd0,0xec},
			{0xd1,0x1 },
			#endif				
			#if VDOREC_18_18FPS
			//Video record daylight 24M 60Hz 18-18FPS maxgain:0x88
			{0xfd,0x1 },
			{0x94,VDOREC_18_18FPS_MAXGAIN},
			{0x9c,VDOREC_18_18FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0xf5},
			{0xf0,0x4a},
			{0xfd,0x1 },
			{0x90,0xbc},
			{0x91,0x1 },
			{0x92,0x4a},
			{0x98,0x4a},
			{0xfd,0x1 },
			{0xcc,VDOREC_18_18FPS_MAXGAIN},
			{0xcd,VDOREC_18_18FPS_MAXGAIN},
			{0xce,0xbc},
			{0xcf,0x1 },
			{0xd0,0xbc},
			{0xd1,0x1 },
			#endif		
			#if VDOREC_16_16FPS
			//Video record daylight 24M 60Hz 16-16FPS maxgain:0x84   
			{0xfd,0x1 },
			{0x94,VDOREC_16_16FPS_MAXGAIN},
			{0x9c,VDOREC_16_16FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x2 },
			{0x0a,0x99},
			{0xf0,0x42},
			{0xfd,0x1 },
			{0x90,0xce},
			{0x91,0x1 },
			{0x92,0x42},
			{0x98,0x42},
			{0xfd,0x1 },
			{0xcc,VDOREC_16_16FPS_MAXGAIN},
			{0xcd,VDOREC_16_16FPS_MAXGAIN},
			{0xce,0xce},
			{0xcf,0x1 },
			{0xd0,0xce},
			{0xd1,0x1 },
			#endif
			#if VDOREC_14_14FPS
			//Video record daylight 24M 60Hz 14-14FPS maxgain:0x80
			{0xfd,0x1 },
			{0x94,VDOREC_14_14FPS_MAXGAIN},
			{0x9c,VDOREC_14_14FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x3 },
			{0x0a,0x6a},
			{0xf0,0x3a},
			{0xfd,0x1 },
			{0x90,0xd0},
			{0x91,0x1 },
			{0x92,0x3a},
			{0x98,0x3a},
			{0xfd,0x1 },
			{0xcc,VDOREC_14_14FPS_MAXGAIN},
			{0xcd,VDOREC_14_14FPS_MAXGAIN},
			{0xce,0xd0},
			{0xcf,0x1 },
			{0xd0,0xd0},
			{0xd1,0x1 },
			#endif
			#if VDOREC_12_12FPS
			//Video record daylight 24M 60Hz 12-12FPS maxgain:0x7c
			{0xfd,0x1 },
			{0x94,VDOREC_12_12FPS_MAXGAIN},
			{0x9c,VDOREC_12_12FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x4 },
			{0x0a,0xa7},
			{0xf0,0x31},
			{0xfd,0x1 },
			{0x90,0xea},
			{0x91,0x1 },
			{0x92,0x31},
			{0x98,0x31},
			{0xfd,0x1 },
			{0xcc,VDOREC_12_12FPS_MAXGAIN},
			{0xcd,VDOREC_12_12FPS_MAXGAIN},
			{0xce,0xea},
			{0xcf,0x1 },
			{0xd0,0xea},
			{0xd1,0x1 },
			#endif	
{0xFF,0xFF},	//־	       
},
// UPCC  mode
{	
      #if VDOREC_20_20FPS
			//Video record daylight 24M 60Hz 20-20FPS maxgain:0x8c
			{0xfd,0x1 },
			{0x94,VDOREC_20_20FPS_MAXGAIN},
			{0x9c,VDOREC_20_20FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0x72},
			{0xf0,0x52},
			{0xfd,0x1 },
			{0x90,0xec},
			{0x91,0x1 },
			{0x92,0x52},
			{0x98,0x52},
			{0xfd,0x1 },
			{0xcc,VDOREC_20_20FPS_MAXGAIN},
			{0xcd,VDOREC_20_20FPS_MAXGAIN},
			{0xce,0xec},
			{0xcf,0x1 },
			{0xd0,0xec},
			{0xd1,0x1 },
			#endif				
			#if VDOREC_18_18FPS
			//Video record daylight 24M 60Hz 18-18FPS maxgain:0x88
			{0xfd,0x1 },
			{0x94,VDOREC_18_18FPS_MAXGAIN},
			{0x9c,VDOREC_18_18FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x1 },
			{0x0a,0xf5},
			{0xf0,0x4a},
			{0xfd,0x1 },
			{0x90,0xbc},
			{0x91,0x1 },
			{0x92,0x4a},
			{0x98,0x4a},
			{0xfd,0x1 },
			{0xcc,VDOREC_18_18FPS_MAXGAIN},
			{0xcd,VDOREC_18_18FPS_MAXGAIN},
			{0xce,0xbc},
			{0xcf,0x1 },
			{0xd0,0xbc},
			{0xd1,0x1 },
			#endif		
			#if VDOREC_16_16FPS
			//Video record daylight 24M 60Hz 16-16FPS maxgain:0x84   
			{0xfd,0x1 },
			{0x94,VDOREC_16_16FPS_MAXGAIN},
			{0x9c,VDOREC_16_16FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x2 },
			{0x0a,0x99},
			{0xf0,0x42},
			{0xfd,0x1 },
			{0x90,0xce},
			{0x91,0x1 },
			{0x92,0x42},
			{0x98,0x42},
			{0xfd,0x1 },
			{0xcc,VDOREC_16_16FPS_MAXGAIN},
			{0xcd,VDOREC_16_16FPS_MAXGAIN},
			{0xce,0xce},
			{0xcf,0x1 },
			{0xd0,0xce},
			{0xd1,0x1 },
			#endif
			#if VDOREC_14_14FPS
			//Video record daylight 24M 60Hz 14-14FPS maxgain:0x80
			{0xfd,0x1 },
			{0x94,VDOREC_14_14FPS_MAXGAIN},
			{0x9c,VDOREC_14_14FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x3 },
			{0x0a,0x6a},
			{0xf0,0x3a},
			{0xfd,0x1 },
			{0x90,0xd0},
			{0x91,0x1 },
			{0x92,0x3a},
			{0x98,0x3a},
			{0xfd,0x1 },
			{0xcc,VDOREC_14_14FPS_MAXGAIN},
			{0xcd,VDOREC_14_14FPS_MAXGAIN},
			{0xce,0xd0},
			{0xcf,0x1 },
			{0xd0,0xd0},
			{0xd1,0x1 },
			#endif
			#if VDOREC_12_12FPS
			//Video record daylight 24M 60Hz 12-12FPS maxgain:0x7c
			{0xfd,0x1 },
			{0x94,VDOREC_12_12FPS_MAXGAIN},
			{0x9c,VDOREC_12_12FPS_MAXGAIN},
			{0xfd,0x0 },
			{0x09,0x4 },
			{0x0a,0xa7},
			{0xf0,0x31},
			{0xfd,0x1 },
			{0x90,0xea},
			{0x91,0x1 },
			{0x92,0x31},
			{0x98,0x31},
			{0xfd,0x1 },
			{0xcc,VDOREC_12_12FPS_MAXGAIN},
			{0xcd,VDOREC_12_12FPS_MAXGAIN},
			{0xce,0xea},
			{0xcf,0x1 },
			{0xd0,0xea},
			{0xd1,0x1 },
			#endif	
{0xFF,0xFF},	//־	     
}  
};


LOCAL uint32 set_SP0A18_video_mode(uint32 mode)
{
    uint8 data=0x00;
    uint16 i;
    SENSOR_REG_T* sensor_reg_ptr = PNULL;
    uint8 tempregval = 0;
    SCI_TRACE_LOW(" xg:set_SP0A18_video_mode ,%d,%d\r\n",mode,Antiflicker);
    SCI_ASSERT(mode <=DCAMERA_MODE_MAX);

	if(Antiflicker ==DCAMERA_FLICKER_50HZ )
	{
	    if(CHIP_DetectMemType())
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_video_mode_nor_tab_50hz[mode];

	    else
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_video_mode_nor_tab_50hz[mode];
	}
	else
	{
	    if(CHIP_DetectMemType())
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_video_mode_nor_tab_60hz[mode];

	    else
	       
	            sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_video_mode_nor_tab_60hz[mode];

	}

    SCI_ASSERT(PNULL != sensor_reg_ptr);

    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value); i++)
    {
    	SP0A18_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }
    return 0;
}

__align(4) const SENSOR_REG_T SP0A18_ev_tab[][10]=
{   
    {
		{0xfd,0x00},   
		{0xed,Pre_Value_P0_0xf7-0x18+0x04},			
		{0xf7,Pre_Value_P0_0xf7-0x18},//level -3
		{0xf8,Pre_Value_P0_0xf8-0x18},	
		{0xec,Pre_Value_P0_0xf8-0x18-0x04},		
		{0xef,Pre_Value_P0_0xf9-0x18+0x04},	
		{0xf9,Pre_Value_P0_0xf9-0x18},
		{0xfa,Pre_Value_P0_0xfa-0x18},		
		{0xee,Pre_Value_P0_0xfa-0x18-0x04},			
		{0xff,0xff},        
	},
    {
		{0xfd,0x00},   
		{0xed,Pre_Value_P0_0xf7-0x10+0x04},			
		{0xf7,Pre_Value_P0_0xf7-0x10},//level -2
		{0xf8,Pre_Value_P0_0xf8-0x10},	
		{0xec,Pre_Value_P0_0xf8-0x10-0x04},		
		{0xef,Pre_Value_P0_0xf9-0x10+0x04},	
		{0xf9,Pre_Value_P0_0xf9-0x10},
		{0xfa,Pre_Value_P0_0xfa-0x10},		
		{0xee,Pre_Value_P0_0xfa-0x10-0x04},			
		{0xff,0xff},      
	},
    {
		{0xfd,0x00},   
		{0xed,Pre_Value_P0_0xf7-0x08+0x04},			
		{0xf7,Pre_Value_P0_0xf7-0x08},//level -1
		{0xf8,Pre_Value_P0_0xf8-0x08},	
		{0xec,Pre_Value_P0_0xf8-0x08-0x04},		
		{0xef,Pre_Value_P0_0xf9-0x08+0x04},	
		{0xf9,Pre_Value_P0_0xf9-0x08},
		{0xfa,Pre_Value_P0_0xfa-0x08},		
		{0xee,Pre_Value_P0_0xfa-0x08-0x04},			
		{0xff,0xff},    
	},
    {
		{0xfd,0x00},   
		{0xed,Pre_Value_P0_0xf7+4},			
		{0xf7,Pre_Value_P0_0xf7},//level 0
		{0xf8,Pre_Value_P0_0xf8},	
		{0xec,Pre_Value_P0_0xf8-4},		
		{0xef,Pre_Value_P0_0xf9+4},	
		{0xf9,Pre_Value_P0_0xf9},
		{0xfa,Pre_Value_P0_0xfa},		
		{0xee,Pre_Value_P0_0xfa-4},			
		{0xff,0xff},    
	},
    {
		{0xfd,0x00},   
		{0xed,Pre_Value_P0_0xf7+0x08+0x04},			
		{0xf7,Pre_Value_P0_0xf7+0x08},//level -1
		{0xf8,Pre_Value_P0_0xf8+0x08},	
		{0xec,Pre_Value_P0_0xf8+0x08-0x04},		
		{0xef,Pre_Value_P0_0xf9+0x08+0x04},	
		{0xf9,Pre_Value_P0_0xf9+0x08},
		{0xfa,Pre_Value_P0_0xfa+0x08},		
		{0xee,Pre_Value_P0_0xfa+0x08-0x04},			
		{0xff,0xff},    
	},
    {
		{0xfd,0x00},   
		{0xed,Pre_Value_P0_0xf7+0x10+0x04},			
		{0xf7,Pre_Value_P0_0xf7+0x10},//level -2
		{0xf8,Pre_Value_P0_0xf8+0x10},	
		{0xec,Pre_Value_P0_0xf8+0x10-0x04},		
		{0xef,Pre_Value_P0_0xf9+0x10+0x04},	
		{0xf9,Pre_Value_P0_0xf9+0x10},
		{0xfa,Pre_Value_P0_0xfa+0x10},		
		{0xee,Pre_Value_P0_0xfa+0x10-0x04},			
		{0xff,0xff},       
	},
    {
		{0xfd,0x00},   
		{0xed,Pre_Value_P0_0xf7+0x18+0x04},			
		{0xf7,Pre_Value_P0_0xf7+0x18},//level -3
		{0xf8,Pre_Value_P0_0xf8+0x18},	
		{0xec,Pre_Value_P0_0xf8+0x18-0x04},		
		{0xef,Pre_Value_P0_0xf9+0x18+0x04},	
		{0xf9,Pre_Value_P0_0xf9+0x18},
		{0xfa,Pre_Value_P0_0xfa+0x18},		
		{0xee,Pre_Value_P0_0xfa+0x18-0x04},			
		{0xff,0xff},    
	},
};

LOCAL uint32 set_SP0A18_ev(uint32 level)
{
    uint16 i; 
    SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)SP0A18_ev_tab[level];

    SCI_ASSERT(PNULL != sensor_reg_ptr);
    SCI_ASSERT(level < 7);
 
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) ||(0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        SP0A18_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }

    
    return 0;
}

LOCAL SENSOR_REG_TAB_INFO_T s_SP0A18_resolution_Tab_YUV[]=
{   
	// COMMON INIT
	{ ADDR_AND_LEN_OF_ARRAY(SP0A18_YVU_640X480),    640,    480, 24,  SENSOR_IMAGE_FORMAT_YUV422 },

	// YUV422 PREVIEW 1
	{ PNULL,                    0,   640,    480,    24   ,        SENSOR_IMAGE_FORMAT_YUV422 },
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      },   


	// YUV422 PREVIEW 2 
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      },   
	{ PNULL,                    0,      0,  0  ,        0,        0      }

};

LOCAL SENSOR_IOCTL_FUNC_TAB_T s_SP0A18_ioctl_func_tab = 
{
// Internal 
    PNULL,
    SP0A18_Power_On,  // PNULL 
    PNULL,
    SP0A18_Identify,
    
    PNULL,          // write register
    PNULL,          // read  register   
    PNULL,
    PNULL,

// External
    set_SP0A18_ae_enable, //set_ae_enable,
    hmirror_enable, //set_hmirror_enable,
    vmirror_enable, //set_vmirror_enable,
    
    set_brightness,
    set_contrast, //write_ev_value,
    PNULL, //read_gain_value,
    PNULL, //write_gain_value,
    set_preview_mode, //read_gain_scale,
    set_image_effect, //set_frame_rate,

    PNULL,//GC0309_BeforeSnapshot, 
    PNULL, //GC0309_After_Snapshot,

    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    PNULL,
    set_SP0A18_awb,    
    PNULL, //read_ev_value,
    PNULL, //write_ev_value,
    set_SP0A18_ev, //read_gain_value,
    PNULL, //write_gain_value,
    PNULL, //read_gain_scale,
    PNULL, //set_frame_rate,
    
    PNULL,
    PNULL,
    set_SP0A18_anti_flicker, //PNULL,
    set_SP0A18_video_mode,    
};

PUBLIC SENSOR_INFO_T g_SP0A18_yuv_info =
{
	SP0A18_I2C_ADDR_W,				// salve i2c write address
	SP0A18_I2C_ADDR_R, 				// salve i2c read address
	
	0,								// bit0: 0: i2c register value is 8 bit, 1: i2c register value is 16 bit
									// bit2: 0: i2c register addr  is 8 bit, 1: i2c register addr  is 16 bit
									// other bit: reseved
	SENSOR_HW_SIGNAL_PCLK_P|\
	SENSOR_HW_SIGNAL_VSYNC_P|\
	SENSOR_HW_SIGNAL_HSYNC_P,		// bit0: 0:negative; 1:positive -> polarily of pixel clock
									// bit2: 0:negative; 1:positive -> polarily of horizontal synchronization signal
									// bit4: 0:negative; 1:positive -> polarily of vertical synchronization signal
									// other bit: reseved											

	// preview mode
	SENSOR_ENVIROMENT_NORMAL|\
	SENSOR_ENVIROMENT_NIGHT|\
	SENSOR_ENVIROMENT_SUNNY,		
	
	// image effect
	SENSOR_IMAGE_EFFECT_NORMAL|\
	SENSOR_IMAGE_EFFECT_BLACKWHITE|\
	SENSOR_IMAGE_EFFECT_RED|\
	SENSOR_IMAGE_EFFECT_GREEN|\
	SENSOR_IMAGE_EFFECT_BLUE|\
	SENSOR_IMAGE_EFFECT_YELLOW|\
	SENSOR_IMAGE_EFFECT_NEGATIVE|\
	SENSOR_IMAGE_EFFECT_CANVAS,
	
	// while balance mode
	0,
		
	7,								// bit[0:7]: count of step in brightness, contrast, sharpness, saturation
									// bit[8:31] reseved
	
	SENSOR_LOW_PULSE_RESET,		// reset pulse level
	100,								// reset pulse width(ms)
	
	SENSOR_HIGH_LEVEL_PWDN,			// 1: high level valid; 0: low level valid
	2,
	0x02, 0x0a,
	0x02, 0x0a,
	
	SENSOR_AVDD_2800MV,				// voltage of avdd	
	
	640,							// max width of source image
	480,							// max height of source image
	"SP0A18",						// name of sensor												

	SENSOR_IMAGE_FORMAT_YUV422,		// define in SENSOR_IMAGE_FORMAT_E enum,
									// if set to SENSOR_IMAGE_FORMAT_MAX here, image format depent on SENSOR_REG_TAB_INFO_T
	SENSOR_IMAGE_PATTERN_YUV422_YUYV,	// pattern of input image form sensor;			

	s_SP0A18_resolution_Tab_YUV,	// point to resolution table information structure
	&s_SP0A18_ioctl_func_tab,		// point to ioctl function table
			
	PNULL,							// information and table about Rawrgb sensor
	PNULL,							// extend information about sensor	
	SENSOR_AVDD_2800MV,                     // iovdd
	SENSOR_AVDD_1800MV,                      // dvdd
	3,								//skip frame num before preview
	0,								//skip frame num before capture
	0,			//skip frame num during preview
	2			//skip frame num during video preview
};

LOCAL uint32 SP0A18_Power_On(uint32 power_on)
{
    SENSOR_AVDD_VAL_E		dvdd_val=g_SP0A18_yuv_info.dvdd_val;
    SENSOR_AVDD_VAL_E		avdd_val=g_SP0A18_yuv_info.avdd_val;
    SENSOR_AVDD_VAL_E		iovdd_val=g_SP0A18_yuv_info.iovdd_val;  
    BOOLEAN 				power_down=g_SP0A18_yuv_info.power_down_level;	    
    BOOLEAN 				reset_level=g_SP0A18_yuv_info.reset_pulse_level;
    uint32 				reset_width=g_SP0A18_yuv_info.reset_pulse_width;	    
    
    if(SCI_TRUE==power_on)
	{
		Sensor_SetVoltage(dvdd_val, avdd_val, iovdd_val);
	        SCI_Sleep(10);	        
#ifdef DOUBLE_CAMERA_SUPPORT
		if (Sensor_GetFrontCameraState())
		{
			GPIO_SetFrontSensorPwdn(!power_down);
			GPIO_SetSensorPwdn(power_down);
		}
		else
		{
			GPIO_SetFrontSensorPwdn(power_down);
			GPIO_SetSensorPwdn(!power_down);
		}
#else      
	        GPIO_SetSensorPwdn(!power_down);
#endif

		//Open Mclk in default frequency
		Sensor_SetMCLK(24);   
             SCI_Sleep(50);
			 
#ifdef SUB_CAMERA_SUPPORT
		//when use back camera, it is better to disable front camera
		Sub_Camera_Disable();
#endif

		//No reset pin on SP0A18
		/*
	        SCI_Sleep(20);
	        Sensor_SetResetLevel(reset_level);
	        SCI_Sleep(reset_width);
	        Sensor_SetResetLevel(!reset_level);
	        SCI_Sleep(100);
	        */
	}
    else
    {
#ifdef DOUBLE_CAMERA_SUPPORT
		GPIO_SetFrontSensorPwdn(power_down);
		GPIO_SetSensorPwdn(power_down);
#else	
		GPIO_SetSensorPwdn(power_down);
#endif

		Sensor_SetMCLK(SENSOR_DISABLE_MCLK);           

		Sensor_SetVoltage(SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED, SENSOR_AVDD_CLOSED);        
			SCI_Sleep(20);
    }
    

    return SCI_SUCCESS;
}

#ifdef SUB_CAMERA_SUPPORT
void Init_Main_Camera_For_Sub(SENSOR_OUTPUT_MODE_E mode)
{
	if(SENSOR_OUTPUT_MODE_SPI == mode)
	{
		SP0A18_WriteReg(0xfd,0x00);
		SP0A18_WriteReg(0x30,0x00);
		SP0A18_WriteReg(0x31,0x00);	//0x10 PCLK inversion
		SP0A18_WriteReg(0x1b,0x05);
		SP0A18_WriteReg(0x1c,0x00);
		SP0A18_WriteReg(0x2e,0x80);
		SP0A18_WriteReg(0x2c,0x0e);
	}
	
        SP0A18_WriteReg(0xfd,0x00);
	SP0A18_WriteReg(0x0e,0xf7);
	SP0A18_WriteReg(0x5f,0x01);
	SP0A18_WriteReg(0x35,0x80);	//swap word
	      
        SP0A18_WriteReg(0x32,0x00);  
        SP0A18_WriteReg(0xfd,0x01);     
        SP0A18_WriteReg(0x28,0xc4);//ac    
        SP0A18_WriteReg(0x29,0x9e);//94   
        SP0A18_WriteReg(0xfd,0x00); 
        SP0A18_WriteReg(0xe7,0x03);       
        SP0A18_WriteReg(0xe7,0x00);       

	
  //special
        SP0A18_WriteReg(0xfd,0x01);
        SP0A18_WriteReg(0x00,0x01);
//  pregain                
        SP0A18_WriteReg(0xc6,0x0a);//a~f
        SP0A18_WriteReg(0xc7,0x0a);//a~f
        SP0A18_WriteReg(0xc8,0x0a);//a~f
        SP0A18_WriteReg(0xc9,0x0a);//a~f
        
// AWB
        SP0A18_WriteReg(0xfd,0x01);
        
        SP0A18_WriteReg(0x10,0x0d);//
        SP0A18_WriteReg(0x11,0x08);//d//11
        SP0A18_WriteReg(0x12,0x08);//d//11
        SP0A18_WriteReg(0x17,0x27);//2f 
        SP0A18_WriteReg(0x18,0x45);  
        SP0A18_WriteReg(0x1a,0xb2);//AC//9e
        SP0A18_WriteReg(0x1b,0xbb);//9e
        SP0A18_WriteReg(0x2a,0xE0);
        SP0A18_WriteReg(0x2b,0x10);//35
        SP0A18_WriteReg(0x2c,0x9f);//90
        SP0A18_WriteReg(0x2d,0x99);
        SP0A18_WriteReg(0x2e,0x99);
        SP0A18_WriteReg(0x6f,0x0a);//5
        SP0A18_WriteReg(0x8b,0x2f);
        
        SP0A18_WriteReg(0x20,0xd0);
        SP0A18_WriteReg(0x21,0x78);
        SP0A18_WriteReg(0x22,0xe0);
        SP0A18_WriteReg(0x23,0x80); 
        SP0A18_WriteReg(0x16,0x08);//14 
//  smooth 
        SP0A18_WriteReg(0xfd,0x00);
        SP0A18_WriteReg(0x4f,0x54);//43 raw_gf_fac_outdoor raw_gf_fac_nr 55
        SP0A18_WriteReg(0x50,0x22);//11 draw_gf_fac_dummy raw_gf_fac_low
        SP0A18_WriteReg(0x57,0x06);//06 raw_dif_thr_outdoor
        SP0A18_WriteReg(0x58,0x0f);//16 raw_dif_thr_normal
        SP0A18_WriteReg(0x56,0x10);//18 dummy
        SP0A18_WriteReg(0x59,0x10);//18 low
         
        SP0A18_WriteReg(0x5a,0x70);// raw_rb_fac_outdoor,raw_rb_fac_nr
        SP0A18_WriteReg(0x5b,0x00);// raw_rb_fac_dummy,raw_rb_fac_low
        
// sharpen  
        SP0A18_WriteReg(0xfd,0x01);
        SP0A18_WriteReg(0xe5,0x20);
        SP0A18_WriteReg(0xe6,0x10);
        SP0A18_WriteReg(0xe7,0x20);     
        SP0A18_WriteReg(0xe8,0x38);//sharp_fac_pos_outdoor
        SP0A18_WriteReg(0xec,0x38);//sharp_fac_neg_outdoor
        SP0A18_WriteReg(0xe9,0x30);//sharp_fac_pos_nr
        SP0A18_WriteReg(0xed,0x30);//sharp_fac_neg_nr
        SP0A18_WriteReg(0xea,0x30);//sharp_fac_pos_dummy
        SP0A18_WriteReg(0xef,0x30);//sharp_fac_neg_dummy
        SP0A18_WriteReg(0xeb,0x10);//sharp_fac_pos_low
        SP0A18_WriteReg(0xf0,0x20);//sharp_fac_neg_low
        
// gamma     
        SP0A18_WriteReg(0xfd,0x00);
        SP0A18_WriteReg(0x8b,0x0 );//0 
        SP0A18_WriteReg(0x8c,0xC );//11
        SP0A18_WriteReg(0x8d,0x19);//19 
        SP0A18_WriteReg(0x8e,0x2C);//28 
        SP0A18_WriteReg(0x8f,0x49);//46 
        SP0A18_WriteReg(0x90,0x61);//61 
        SP0A18_WriteReg(0x91,0x77);//78 
        SP0A18_WriteReg(0x92,0x8A);//8A 
        SP0A18_WriteReg(0x93,0x9B);//9B 
        SP0A18_WriteReg(0x94,0xA9);//A9 
        SP0A18_WriteReg(0x95,0xB5);//B5 
        SP0A18_WriteReg(0x96,0xC0);//C0 
        SP0A18_WriteReg(0x97,0xCA);//CA 
        SP0A18_WriteReg(0x98,0xD4);//D4 
        SP0A18_WriteReg(0x99,0xDD);//DD 
        SP0A18_WriteReg(0x9a,0xE6);//E6 
        SP0A18_WriteReg(0x9b,0xEF);//EF 
        SP0A18_WriteReg(0xfd,0x01);//01 
        SP0A18_WriteReg(0x8d,0xF7);//F7 
        SP0A18_WriteReg(0x8e,0xFF);//FF 
        
             
// uv en    
        SP0A18_WriteReg(0xfd,0x00); 
        SP0A18_WriteReg(0xb0,0x00);//uv_dif_gain_outdoor,uv_dif_gain_nr
        SP0A18_WriteReg(0xb1,0x33);//uv_dif_gain_dummy,uv_dif_gain_low   
        
//  neg    
        SP0A18_WriteReg(0xfd,0x01);
        SP0A18_WriteReg(0xd2,0x00);  
        
//  lowlight lum
        SP0A18_WriteReg(0xfd,0x00); 
        SP0A18_WriteReg(0xb2,0x08);//lum_limit
        SP0A18_WriteReg(0xb3,0x1f);//lum_set
        SP0A18_WriteReg(0xb4,0x20);//black_vt
        SP0A18_WriteReg(0xb5,0x45);//white_vt
        
//  auto sat  
        SP0A18_WriteReg(0xfd,0x00);
        SP0A18_WriteReg(0xbf,0x01);//autosa_en
        SP0A18_WriteReg(0xc0,0xff);//a0//y_p_th
        SP0A18_WriteReg(0xc1,0xff);//a0//y_mean_th
        
//  saturaion 
        SP0A18_WriteReg(0xfd,0x00); 
        SP0A18_WriteReg(0xd8,0x80);//74//88sat_uv_s1
        SP0A18_WriteReg(0xd9,0x80);//74//88sat_uv_s2
        SP0A18_WriteReg(0xda,0x80);//74//88sat_uv_s3
        SP0A18_WriteReg(0xdb,0x80);//74//88sat_uv_s4
        
//  heq      
        SP0A18_WriteReg(0xfd,0x00);
        SP0A18_WriteReg(0xdc,0x00);//heq_offset
        SP0A18_WriteReg(0xdd,0x80);////ku
        SP0A18_WriteReg(0xde,0xa8);//b0//kl
        SP0A18_WriteReg(0xdf,0x70);//heq_mean
        
//  CCM   
        SP0A18_WriteReg(0xfd,0x01);
        SP0A18_WriteReg(0xa0,0x80);//80(ɫӽɫ)
        SP0A18_WriteReg(0xa1,0x0 );//0 
        SP0A18_WriteReg(0xa2,0x0 );//0 
        SP0A18_WriteReg(0xa3,0xf3);//f0
        SP0A18_WriteReg(0xa4,0x8e);//a6
        SP0A18_WriteReg(0xa5,0x0 );//ea
        SP0A18_WriteReg(0xa6,0x0 );//0 
        SP0A18_WriteReg(0xa7,0xe6);//e6
        SP0A18_WriteReg(0xa8,0x9a);//9a
        SP0A18_WriteReg(0xa9,0x0 );//0 
        SP0A18_WriteReg(0xaa,0x3 );//33
        SP0A18_WriteReg(0xab,0xc );//c 
        
//  skin    
        SP0A18_WriteReg(0xfd,0x01);
        
        SP0A18_WriteReg(0x8b,0x2f);//bit7,0 en
        SP0A18_WriteReg(0x01,0xf0);
        SP0A18_WriteReg(0x02,0x04);
        SP0A18_WriteReg(0x03,0x70);
        SP0A18_WriteReg(0x04,0x99);//88
        SP0A18_WriteReg(0x05,0x24);//13
// resize en  
        SP0A18_WriteReg(0xfd,0x01);
        SP0A18_WriteReg(0x0e,0x00);
	
//window 240*320
        SP0A18_WriteReg(0xfd,0x00);	
	SP0A18_WriteReg(0x47,0x00);
	SP0A18_WriteReg(0x48,0x00);
	SP0A18_WriteReg(0x49,0x01);
	SP0A18_WriteReg(0x4a,0x40);
	SP0A18_WriteReg(0x4b,0x00);
	SP0A18_WriteReg(0x4c,0x00);
	SP0A18_WriteReg(0x4d,0x00);
	SP0A18_WriteReg(0x4e,0xf0);
	
	SP0A18_WriteReg(0x32,0x08);
	
}

uint32 Set_Main_Camera_Brightness(uint32 level)
{
	return set_brightness(level);
}

uint32 Set_Main_Camera_Contrast(uint32 level)
{
	return set_contrast(level);
}

uint32 Set_Main_Camera_Image_Effect(uint32 effect_type)
{
	return set_image_effect(effect_type);
}

__align(4) const SENSOR_REG_T sub_cam_awb_tab[][7]=
{
    //AUTO  // AUTO 3000K~7000K 
	   	{
			{0xfd,0x01},                      
			{0x28,Pre_Value_P1_0x28},		                  
			{0x29,Pre_Value_P1_0x29},                      
			{0xfd,0x00}, 		
			{0x32,0x08},   
			{0xfd,0x00},  
		    {0xff,0xff} 
		},    
		//INCANDESCENCE:  //2800K~3000K  
		{
			{0xfd,0x00}, 
			{0x32,0x00},                 
			{0xfd,0x01},                 
			{0x28,0x7b},		             
			{0x29,0xd3},		             
			{0xfd,0x00}, 
			{0xff,0xff} 
		},
		//U30 //3000K
		{
			{0xfd,0x00},
			{0x32,0x00},       
			{0xfd,0x01},       
			{0x28,0x82},	     
			{0x29,0xe2},	     
			{0xfd,0x00},   	
			{0xff,0xff} 
		},  
		//CWF  //4150K
		{
			{0xfd,0x00},
			{0x32,0x00},       
			{0xfd,0x01},       
			{0x28,0xa3},	     
			{0x29,0xb3},	     
			{0xfd,0x00}, 		
			{0xff,0xff} 
		},    
		//FLUORESCENT://4200K~5000K   
		{
			{0xfd,0x00},  
			{0x32,0x00},                  
			{0xfd,0x01},                  
			{0x28,0xb4},		              
			{0x29,0xc4},		              
			{0xfd,0x00},		
		    {0xff,0xff} 
		},
		//SUN: //DAYLIGHT//6500K   
		{
			{0xfd,0x00}, 
			{0x32,0x00},           
			{0xfd,0x01},           
			{0x28,0xc1},	         
			{0x29,0x88},	         
			{0xfd,0x00}, 		
		    {0xff,0xff} 
		},
                //CLOUD: //7000K 
		{
			{0xfd,0x00}, 
			{0x32,0x00},          
			{0xfd,0x01},          
			{0x28,0xe2},	        
			{0x29,0x82},	        
			{0xfd,0x00},		
		    {0xff,0xff} 
		},	
};

LOCAL void SP0A18_Write_Group_Regs( SENSOR_REG_T* sensor_reg_ptr )
{
    uint32 i;
    
    for(i = 0; (0xFF != sensor_reg_ptr[i].reg_addr) || (0xFF != sensor_reg_ptr[i].reg_value) ; i++)
    {
        //Sensor_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
		SP0A18_WriteReg(sensor_reg_ptr[i].reg_addr, sensor_reg_ptr[i].reg_value);
    }

}

uint32 Set_Main_Camera_AWB(uint32 mode)
{
	SENSOR_REG_T* sensor_reg_ptr = (SENSOR_REG_T*)sub_cam_awb_tab[mode];

	SCI_ASSERT(mode < 7);
	SCI_ASSERT(PNULL != sensor_reg_ptr);
	
	SP0A18_Write_Group_Regs(sensor_reg_ptr);
	
	SCI_TRACE_LOW("set_Main_Camera_awb_mode: mode = %d", mode);
	
	return 0;
}


#endif

LOCAL uint32 hmirror_enable(uint32 enable)
{
#ifdef DOUBLE_CAMERA_SUPPORT
uint32 i;
BOOLEAN is_page0 = FALSE;
for(i = 0; i < s_SP0A18_resolution_Tab_YUV->reg_count; i++)
{
if (s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0xfd)
{
if (s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x00)
is_page0 = TRUE;
else if (s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x01)
is_page0 = FALSE;
}
if ((s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0x31) && is_page0)
{
if(enable)
{
s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value |= 0x20;
}
else
{
s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value &= ~0x20;
}
break;
}
}
return SCI_SUCCESS; 
#endif
}

LOCAL uint32 vmirror_enable(uint32 enable)
{
#ifdef DOUBLE_CAMERA_SUPPORT
uint32 i;
BOOLEAN is_page0 = FALSE;
for(i = 0; i < s_SP0A18_resolution_Tab_YUV->reg_count; i++)
{
if (s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0xfd)
{
if (s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x00)
is_page0 = TRUE;
else if (s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value == 0x01)
is_page0 = FALSE;
}
if ((s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_addr == 0x31) && is_page0)
{
if(enable)
{
s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value |= 0x40;
}
else
{
s_SP0A18_resolution_Tab_YUV->sensor_reg_tab_ptr[i].reg_value &= ~0x40;
}
break;
}
}
return SCI_SUCCESS; 
#endif
}

/***************************************
ATC function used by AT command for FAE
easy to check the effect of single reg value

BESURE WE HAVE REVISE PLATFORM TO SUPPORT THIS
use ate1 to show command
use at+sdatarxmd=A,B,C to write or read reg
A  1:write  0:read
B  0:select camera  other:reg address 
C  if(B==0)  0:main camera
                  1:sub camera
    if(B!=0)  value
NOTE THESE PARAMS ARE  DECIMAL

***************************************/
void ATC_WriteReg(uint8 add, uint8 val)
{
	if(0 == add)
		g_isMain = !val;
	else if(g_isMain)
		SP0A18_WriteReg(add, val);
	else
		ATC27_WriteReg(add, val);
}
uint8 ATC_ReadReg(uint8 add)
{
	if(g_isMain)
		return SP0A18_ReadReg(add);
	else
		return ATC27_ReadReg(add);
}

#endif




